@@ -6,6 +6,7 @@
 #include <stddef.h>
 #include <stdarg.h>
 #include <time.h>
+#include <xmlrpc-c/c_util.h>
 #include <xmlrpc-c/util.h>
 #include <xmlrpc-c/config.h>
   /* Defines XMLRPC_HAVE_WCHAR, XMLRPC_INT64, XMLRPC_HAVE_TIMEVAL */
@@ -22,14 +23,46 @@
 extern "C" {
 #endif
 
+/*
+  XMLRPC_LIB_EXPORTED marks a symbol in this file that is exported from
+  libxmlrpc.
+
+  XMLRPC_BUILDING_LIB says this compilation is part of libxmlrpc, as
+  opposed to something that _uses_ libxmlrpc.
+*/
+#ifdef XMLRPC_BUILDING_LIB
+#define XMLRPC_LIB_EXPORTED XMLRPC_DLLEXPORT
+#else
+#define XMLRPC_LIB_EXPORTED
+#endif
+
+/*=========================================================================
+**  Global static library initialization
+**=======================================================================*/
+
+void
+xmlrpc_init(xmlrpc_env * const envP);
+
+void
+xmlrpc_term(void);
 
 /*=========================================================================
 **  Version of libxmlrpc
 **=======================================================================*/
+
+/* These are for backward compatibility -- they can't be exported from a
+   Windows DLL.  xmlrpc_server_version() is preferred.
+*/
 extern unsigned int const xmlrpc_version_major;
 extern unsigned int const xmlrpc_version_minor;
 extern unsigned int const xmlrpc_version_point;
 
+XMLRPC_LIB_EXPORTED
+void
+xmlrpc_version(unsigned int * const majorP,
+               unsigned int * const minorP,
+               unsigned int * const pointP);
+
 /*=========================================================================
 **  C types equivalent to XML-RPC types
 **=======================================================================*/
@@ -54,6 +87,19 @@ typedef double xmlrpc_double;
        so it's probably clearer just to use that.  This typedef is here 
        for mathematical completeness.
     */
+typedef struct {
+    /* A datetime of the type defined by XML-RPC <dateTime.iso8601> with
+       a few extensions.  I.e. in the range 1-9999 AD with microsecond
+       resolution.
+    */
+        unsigned int Y;   /* 1-? */
+        unsigned int M;   /* 1-12 */
+        unsigned int D;   /* 1-31 */
+        unsigned int h;   /* 0-23 */
+        unsigned int m;   /* 0-59 */
+        unsigned int s;   /* 0-59 */
+        unsigned int u;   /* 0-999999 */
+} xmlrpc_datetime;
 
 /* xmlrpc_socket is just for backward compatibility, in case someone decided
    to use this in user code.  New code should use the native type for a
@@ -93,12 +139,13 @@ typedef enum {
 
 #define XMLRPC_HAVE_I8 1
 
-/* These are *always* allocated on the heap. No exceptions. */
 typedef struct _xmlrpc_value xmlrpc_value;
 
+XMLRPC_LIB_EXPORTED
 const char *
 xmlrpc_type_name(xmlrpc_type const type);
 
+XMLRPC_LIB_EXPORTED
 void
 xmlrpc_abort_if_array_bad(xmlrpc_value * const arrayP);
 
@@ -106,81 +153,110 @@ xmlrpc_abort_if_array_bad(xmlrpc_value * const arrayP);
     xmlrpc_abort_if_array_bad(val)
 
 /* Increment the reference count of an xmlrpc_value. */
-extern void xmlrpc_INCREF (xmlrpc_value* const value);
+XMLRPC_LIB_EXPORTED
+extern void xmlrpc_INCREF(xmlrpc_value* const value);
 
 /* Decrement the reference count of an xmlrpc_value. If there
 ** are no more references, free it. */
-extern void xmlrpc_DECREF (xmlrpc_value* const value);
+XMLRPC_LIB_EXPORTED
+extern void xmlrpc_DECREF(xmlrpc_value* const value);
 
 /* Get the type of an XML-RPC value. */
+XMLRPC_LIB_EXPORTED
 extern xmlrpc_type xmlrpc_value_type (xmlrpc_value* const value);
 
+XMLRPC_LIB_EXPORTED
 xmlrpc_value *
 xmlrpc_int_new(xmlrpc_env * const envP,
                int          const intValue);
 
-xmlrpc_value *
-xmlrpc_i8_new(xmlrpc_env * const envP, 
-              xmlrpc_int64 const value);
-
+XMLRPC_LIB_EXPORTED
 void 
 xmlrpc_read_int(xmlrpc_env *         const envP,
                 const xmlrpc_value * const valueP,
                 int *                const intValueP);
 
+XMLRPC_LIB_EXPORTED
+xmlrpc_value *
+xmlrpc_i8_new(xmlrpc_env * const envP, 
+              xmlrpc_int64 const value);
+
+XMLRPC_LIB_EXPORTED
+void 
+xmlrpc_read_i8(xmlrpc_env *         const envP,
+               const xmlrpc_value * const valueP,
+               xmlrpc_int64 *       const intValueP);
+
+XMLRPC_LIB_EXPORTED
 xmlrpc_value *
 xmlrpc_bool_new(xmlrpc_env * const envP,
                 xmlrpc_bool  const boolValue);
 
+XMLRPC_LIB_EXPORTED
 void
 xmlrpc_read_bool(xmlrpc_env *         const envP,
                  const xmlrpc_value * const valueP,
                  xmlrpc_bool *        const boolValueP);
 
+XMLRPC_LIB_EXPORTED
 xmlrpc_value *
 xmlrpc_double_new(xmlrpc_env * const envP,
                   double       const doubleValue);
 
+XMLRPC_LIB_EXPORTED
 void
 xmlrpc_read_double(xmlrpc_env *         const envP,
                    const xmlrpc_value * const valueP,
                    xmlrpc_double *      const doubleValueP);
 
+XMLRPC_LIB_EXPORTED
+xmlrpc_value *
+xmlrpc_datetime_new(xmlrpc_env *    const envP, 
+                    xmlrpc_datetime const dt);
+
+XMLRPC_LIB_EXPORTED
 xmlrpc_value *
 xmlrpc_datetime_new_str(xmlrpc_env * const envP,
                         const char * const value);
 
+XMLRPC_LIB_EXPORTED
 xmlrpc_value *
 xmlrpc_datetime_new_sec(xmlrpc_env * const envP, 
                         time_t       const value);
 
+XMLRPC_LIB_EXPORTED
 xmlrpc_value*
 xmlrpc_datetime_new_usec(xmlrpc_env * const envP,
                          time_t       const secs,
                          unsigned int const usecs);
 
 #if XMLRPC_HAVE_TIMEVAL
+XMLRPC_LIB_EXPORTED
 xmlrpc_value *
 xmlrpc_datetime_new_timeval(xmlrpc_env *   const envP, 
                             struct timeval const value);
 #endif
 
 #if XMLRPC_HAVE_TIMESPEC
+XMLRPC_LIB_EXPORTED
 xmlrpc_value *
 xmlrpc_datetime_new_timespec(xmlrpc_env *    const envP, 
                              struct timespec const value);
 #endif
 
 void
-xmlrpc_read_datetime_str(xmlrpc_env *         const envP,
-                         const xmlrpc_value * const valueP,
-                         const char **        const stringValueP);
+XMLRPC_LIB_EXPORTED
+xmlrpc_read_datetime(xmlrpc_env *         const envP,
+                     const xmlrpc_value * const valueP,
+                     xmlrpc_datetime *    const dtP);
 
+XMLRPC_LIB_EXPORTED
 void
 xmlrpc_read_datetime_sec(xmlrpc_env *         const envP,
                          const xmlrpc_value * const valueP,
                          time_t *             const timeValueP);
 
+XMLRPC_LIB_EXPORTED
 void
 xmlrpc_read_datetime_usec(xmlrpc_env *         const envP,
                           const xmlrpc_value * const valueP,
@@ -188,6 +264,7 @@ xmlrpc_read_datetime_usec(xmlrpc_env *         const envP,
                           unsigned int *       const usecsP);
 
 #if XMLRPC_HAVE_TIMEVAL
+XMLRPC_LIB_EXPORTED
 void
 xmlrpc_read_datetime_timeval(xmlrpc_env *         const envP,
                              const xmlrpc_value * const valueP,
@@ -195,57 +272,80 @@ xmlrpc_read_datetime_timeval(xmlrpc_env *         const envP,
 #endif
 
 #if XMLRPC_HAVE_TIMESPEC
+XMLRPC_LIB_EXPORTED
 void
 xmlrpc_read_datetime_timespec(xmlrpc_env *         const envP,
                               const xmlrpc_value * const valueP,
                               struct timespec *    const timeValueP);
 #endif
 
+void
+XMLRPC_LIB_EXPORTED
+xmlrpc_read_datetime_8601(xmlrpc_env *         const envP,
+                          const xmlrpc_value * const valueP,
+                          const char **        const iso8601ValueP);
+
+XMLRPC_LIB_EXPORTED
+void
+xmlrpc_read_datetime_str(xmlrpc_env *         const envP,
+                         const xmlrpc_value * const valueP,
+                         const char **        const stringValueP);
+
+XMLRPC_LIB_EXPORTED
 xmlrpc_value *
 xmlrpc_string_new(xmlrpc_env * const envP,
                   const char * const stringValue);
 
+XMLRPC_LIB_EXPORTED
 xmlrpc_value *
 xmlrpc_string_new_lp(xmlrpc_env * const envP, 
                      size_t       const length,
                      const char * const stringValue);
 
+XMLRPC_LIB_EXPORTED
 xmlrpc_value *
 xmlrpc_string_new_va(xmlrpc_env * const envP,
                      const char * const format,
                      va_list            args);
 
+XMLRPC_LIB_EXPORTED
 xmlrpc_value *
 xmlrpc_string_new_f(xmlrpc_env * const envP,
                     const char * const format,
                     ...);
 
+XMLRPC_LIB_EXPORTED
 xmlrpc_value *
 xmlrpc_string_new_lp_cr(xmlrpc_env * const envP, 
                         size_t       const length,
                         const char * const value);
 
+XMLRPC_LIB_EXPORTED
 xmlrpc_value *
 xmlrpc_string_new_cr(xmlrpc_env * const envP,
                      const char * const value);
 
+XMLRPC_LIB_EXPORTED
 void
 xmlrpc_read_string(xmlrpc_env *         const envP,
                    const xmlrpc_value * const valueP,
                    const char **        const stringValueP);
 
 
+XMLRPC_LIB_EXPORTED
 void
 xmlrpc_read_string_crlf(xmlrpc_env *         const envP,
                         const xmlrpc_value * const valueP,
                         const char **        const stringValueP);
 
+XMLRPC_LIB_EXPORTED
 void
 xmlrpc_read_string_lp_crlf(xmlrpc_env *         const envP,
                            const xmlrpc_value * const valueP,
                            size_t *             const lengthP,
                            const char **        const stringValueP);
 
+XMLRPC_LIB_EXPORTED
 void
 xmlrpc_read_string_lp(xmlrpc_env *         const envP,
                       const xmlrpc_value * const valueP,
@@ -253,80 +353,93 @@ xmlrpc_read_string_lp(xmlrpc_env *         const envP,
                       const char **        const stringValueP);
 
 #if XMLRPC_HAVE_WCHAR
+XMLRPC_LIB_EXPORTED
 xmlrpc_value *
 xmlrpc_string_w_new(xmlrpc_env *    const envP,
                     const wchar_t * const stringValue);
 
+XMLRPC_LIB_EXPORTED
 xmlrpc_value *
 xmlrpc_string_w_new_lp(xmlrpc_env *    const envP, 
                        size_t          const length,
                        const wchar_t * const stringValue);
 
+XMLRPC_LIB_EXPORTED
 void
 xmlrpc_read_string_w(xmlrpc_env *     const envP,
                      xmlrpc_value *   const valueP,
                      const wchar_t ** const stringValueP);
 
+XMLRPC_LIB_EXPORTED
 void
 xmlrpc_read_string_w_crlf(xmlrpc_env *     const envP,
                           xmlrpc_value *   const valueP,
                           const wchar_t ** const stringValueP);
 
+XMLRPC_LIB_EXPORTED
 void
 xmlrpc_read_string_w_lp(xmlrpc_env *     const envP,
                         xmlrpc_value *   const valueP,
                         size_t *         const lengthP,
                         const wchar_t ** const stringValueP);
 
+XMLRPC_LIB_EXPORTED
 void
 xmlrpc_read_string_w_lp_crlf(xmlrpc_env *     const envP,
                              xmlrpc_value *   const valueP,
                              size_t *         const lengthP,
                              const wchar_t ** const stringValueP);
 
+XMLRPC_LIB_EXPORTED
 xmlrpc_value *
 xmlrpc_string_w_new_lp_cr(xmlrpc_env *    const envP, 
                           size_t          const length,
                           const wchar_t * const value);
 
+XMLRPC_LIB_EXPORTED
 xmlrpc_value *
 xmlrpc_string_w_new_cr(xmlrpc_env *    const envP,
                        const wchar_t * const value);
 
 #endif /* XMLRPC_HAVE_WCHAR */
 
+XMLRPC_LIB_EXPORTED
 xmlrpc_value *
 xmlrpc_base64_new(xmlrpc_env *          const envP, 
                   size_t                const length,
                   const unsigned char * const value);
 
+XMLRPC_LIB_EXPORTED
 void
 xmlrpc_read_base64(xmlrpc_env *           const envP,
                    const xmlrpc_value *   const valueP,
                    size_t *               const lengthP,
                    const unsigned char ** const bytestringValueP);
 
+XMLRPC_LIB_EXPORTED
 void
 xmlrpc_read_base64_size(xmlrpc_env *           const envP,
                         const xmlrpc_value *   const valueP,
                         size_t *               const lengthP);
 
+XMLRPC_LIB_EXPORTED
 xmlrpc_value *
 xmlrpc_array_new(xmlrpc_env * const envP);
 
 /* Return the number of elements in an XML-RPC array.
 ** Sets XMLRPC_TYPE_ERROR if 'array' is not an array. */
+XMLRPC_LIB_EXPORTED
 int 
 xmlrpc_array_size(xmlrpc_env *         const env, 
                   const xmlrpc_value * const array);
 
-/* Append an item to an XML-RPC array.
-** Sets XMLRPC_TYPE_ERROR if 'array' is not an array. */
-extern void
-xmlrpc_array_append_item (xmlrpc_env   * const envP,
-                          xmlrpc_value * const arrayP,
-                          xmlrpc_value * const valueP);
+XMLRPC_LIB_EXPORTED
+void
+xmlrpc_array_append_item(xmlrpc_env   * const envP,
+                         xmlrpc_value * const arrayP,
+                         xmlrpc_value * const valueP);
 
+XMLRPC_LIB_EXPORTED
 void
 xmlrpc_array_read_item(xmlrpc_env *         const envP,
                        const xmlrpc_value * const arrayP,
@@ -340,30 +453,35 @@ xmlrpc_array_read_item(xmlrpc_env *         const envP,
    Sets XMLRPC_TYPE_ERROR if 'array' is not an array.
    Sets XMLRPC_INDEX_ERROR if 'index' is out of bounds.
 */
+XMLRPC_LIB_EXPORTED
 xmlrpc_value * 
 xmlrpc_array_get_item(xmlrpc_env *         const envP,
                       const xmlrpc_value * const arrayP,
                       int                  const index);
 
 /* Not implemented--we don't need it yet.
-extern 
-int xmlrpc_array_set_item (xmlrpc_env* env,
-xmlrpc_value* array,
-int index,
-                                  xmlrpc_value* value);
+XMLRPC_LIB_EXPORTED
+int
+xmlrpc_array_set_item(xmlrpc_env *   const envP,
+                      xmlrpc_value * const arrayP,
+                      unsigned int   const index,
+                      xmlrpc_value * const valueP);
 */
 
+XMLRPC_LIB_EXPORTED
 xmlrpc_value *
 xmlrpc_struct_new(xmlrpc_env * const env);
 
 /* Return the number of key/value pairs in a struct.
 ** Sets XMLRPC_TYPE_ERROR if 'strct' is not a struct. */
+XMLRPC_LIB_EXPORTED
 int
-xmlrpc_struct_size (xmlrpc_env   * env, 
-                    xmlrpc_value * strct);
+xmlrpc_struct_size (xmlrpc_env   * const env, 
+                    xmlrpc_value * const strct);
 
 /* Returns true iff 'strct' contains 'key'.
 ** Sets XMLRPC_TYPE_ERROR if 'strct' is not a struct. */
+XMLRPC_LIB_EXPORTED
 int 
 xmlrpc_struct_has_key(xmlrpc_env *   const envP,
                       xmlrpc_value * const strctP,
@@ -373,6 +491,7 @@ xmlrpc_struct_has_key(xmlrpc_env *   const envP,
    Deprecated.  xmlrpc_struct_get_value_v() is more general, and this
    case is not common enough to warrant a shortcut.
 */
+XMLRPC_LIB_EXPORTED
 int 
 xmlrpc_struct_has_key_n(xmlrpc_env   * const envP,
                         xmlrpc_value * const strctP,
@@ -381,6 +500,7 @@ xmlrpc_struct_has_key_n(xmlrpc_env   * const envP,
 
 #if 0
 /* Not implemented yet, but needed for completeness. */
+XMLRPC_LIB_EXPORTED
 int
 xmlrpc_struct_has_key_v(xmlrpc_env *   env, 
                         xmlrpc_value * strct,
@@ -388,6 +508,7 @@ xmlrpc_struct_has_key_v(xmlrpc_env *   env,
 #endif
 
 
+XMLRPC_LIB_EXPORTED
 void
 xmlrpc_struct_find_value(xmlrpc_env *    const envP,
                          xmlrpc_value *  const structP,
@@ -395,18 +516,21 @@ xmlrpc_struct_find_value(xmlrpc_env *    const envP,
                          xmlrpc_value ** const valuePP);
 
 
+XMLRPC_LIB_EXPORTED
 void
 xmlrpc_struct_find_value_v(xmlrpc_env *    const envP,
                            xmlrpc_value *  const structP,
                            xmlrpc_value *  const keyP,
                            xmlrpc_value ** const valuePP);
 
+XMLRPC_LIB_EXPORTED
 void
 xmlrpc_struct_read_value(xmlrpc_env *    const envP,
                          xmlrpc_value *  const structP,
                          const char *    const key,
                          xmlrpc_value ** const valuePP);
 
+XMLRPC_LIB_EXPORTED
 void
 xmlrpc_struct_read_value_v(xmlrpc_env *    const envP,
                            xmlrpc_value *  const structP,
@@ -416,6 +540,7 @@ xmlrpc_struct_read_value_v(xmlrpc_env *    const envP,
 /* The "get_value" functions are deprecated.  Use the "find_value"
    and "read_value" functions instead.
 */
+XMLRPC_LIB_EXPORTED
 xmlrpc_value * 
 xmlrpc_struct_get_value(xmlrpc_env *   const envP,
                         xmlrpc_value * const strctP,
@@ -425,6 +550,7 @@ xmlrpc_struct_get_value(xmlrpc_env *   const envP,
    Deprecated.  xmlrpc_struct_get_value_v() is more general, and this
    case is not common enough to warrant a shortcut.
 */
+XMLRPC_LIB_EXPORTED
 xmlrpc_value * 
 xmlrpc_struct_get_value_n(xmlrpc_env *   const envP,
                           xmlrpc_value * const strctP,
@@ -434,6 +560,7 @@ xmlrpc_struct_get_value_n(xmlrpc_env *   const envP,
 /* Set the value associated with 'key' in 'strct' to 'value'.
    Sets XMLRPC_TYPE_ERROR if 'strct' is not a struct. 
 */
+XMLRPC_LIB_EXPORTED
 void 
 xmlrpc_struct_set_value(xmlrpc_env *   const env,
                         xmlrpc_value * const strct,
@@ -444,6 +571,7 @@ xmlrpc_struct_set_value(xmlrpc_env *   const env,
    The general way to set a structure value is xmlrpc_struct_set_value_v(),
    and this case is not common enough to deserve a shortcut.
 */
+XMLRPC_LIB_EXPORTED
 void 
 xmlrpc_struct_set_value_n(xmlrpc_env *    const env,
                           xmlrpc_value *  const strct,
@@ -453,6 +581,7 @@ xmlrpc_struct_set_value_n(xmlrpc_env *    const env,
 
 /* The same as above, but the key must be an XML-RPC string.
 ** Fails with XMLRPC_TYPE_ERROR if 'keyval' is not a string. */
+XMLRPC_LIB_EXPORTED
 void 
 xmlrpc_struct_set_value_v(xmlrpc_env *   const env,
                           xmlrpc_value * const strct,
@@ -464,6 +593,7 @@ xmlrpc_struct_set_value_v(xmlrpc_env *   const env,
 ** Fails with XMLRPC_TYPE_ERROR if 'struct' is not a struct.
 ** Fails with XMLRPC_INDEX_ERROR if 'index' is out of bounds. */
 
+XMLRPC_LIB_EXPORTED
 void 
 xmlrpc_struct_read_member(xmlrpc_env *    const envP,
                           xmlrpc_value *  const structP,
@@ -477,6 +607,7 @@ xmlrpc_struct_read_member(xmlrpc_env *    const envP,
 
    Deprecated.  Use xmlrpc_struct_read_member() instead.
 */
+XMLRPC_LIB_EXPORTED
 void
 xmlrpc_struct_get_key_and_value(xmlrpc_env *    const env,
                                 xmlrpc_value *  const strct,
@@ -484,38 +615,55 @@ xmlrpc_struct_get_key_and_value(xmlrpc_env *    const env,
                                 xmlrpc_value ** const out_keyval,
                                 xmlrpc_value ** const out_value);
 
+/* The "C pointer" type has no relation to XML-RPC.  It is here for the
+   convenience of programs that use xmlrpc_value for XML-RPC purposes
+   and can benefit from using it for non-XML-RPC purposes as well.
+
+   Also, some people use libxmlrpc for xmlrpc_value alone, because sometimes
+   you need to work with basic data types in richer ways than the C types
+   (int, time_t, etc) allow.
+*/
+
+XMLRPC_LIB_EXPORTED
+xmlrpc_value *
+xmlrpc_cptr_new(xmlrpc_env * const envP,
+                void *       const value);
+
+typedef void (*xmlrpc_cptr_dtor_fn)(void *, void *);
+
+XMLRPC_LIB_EXPORTED
+xmlrpc_value *
+xmlrpc_cptr_new_dtor(xmlrpc_env *        const envP,
+                     void *              const value,
+                     xmlrpc_cptr_dtor_fn const dtor,
+                     void *              const dtorContext);
+
+XMLRPC_LIB_EXPORTED
 void
 xmlrpc_read_cptr(xmlrpc_env *         const envP,
                  const xmlrpc_value * const valueP,
                  void **              const ptrValueP);
 
+XMLRPC_LIB_EXPORTED
 void
 xmlrpc_read_nil(xmlrpc_env *   const envP,
                 xmlrpc_value * const valueP);
                 
-
-void 
-xmlrpc_read_i8(xmlrpc_env *         const envP,
-               const xmlrpc_value * const valueP,
-               xmlrpc_int64 *       const intValueP);
-
-
-xmlrpc_value *
-xmlrpc_cptr_new(xmlrpc_env * const envP,
-                void *       const value);
-
+XMLRPC_LIB_EXPORTED
 xmlrpc_value *
 xmlrpc_nil_new(xmlrpc_env * const envP);
 
 
 /* Build an xmlrpc_value from a format string. */
 
+XMLRPC_LIB_EXPORTED
 xmlrpc_value * 
 xmlrpc_build_value(xmlrpc_env * const env,
                    const char * const format, 
                    ...);
 
 /* The same as the above, but using a va_list and more general */
+XMLRPC_LIB_EXPORTED
 void
 xmlrpc_build_value_va(xmlrpc_env *    const env,
                       const char *    const format,
@@ -523,12 +671,14 @@ xmlrpc_build_value_va(xmlrpc_env *    const env,
                       xmlrpc_value ** const valPP,
                       const char **   const tailP);
 
+XMLRPC_LIB_EXPORTED
 void 
 xmlrpc_decompose_value(xmlrpc_env *   const envP,
                        xmlrpc_value * const value,
                        const char *   const format, 
                        ...);
 
+XMLRPC_LIB_EXPORTED
 void 
 xmlrpc_decompose_value_va(xmlrpc_env *   const envP,
                           xmlrpc_value * const value,
@@ -542,6 +692,7 @@ xmlrpc_decompose_value_va(xmlrpc_env *   const envP,
 
    These are deprecated.  Use xmlrpc_decompose_value... instead.
 */
+XMLRPC_LIB_EXPORTED
 void 
 xmlrpc_parse_value(xmlrpc_env *   const envP,
                    xmlrpc_value * const value,
@@ -549,6 +700,7 @@ xmlrpc_parse_value(xmlrpc_env *   const envP,
                    ...);
 
 /* The same as the above, but using a va_list. */
+XMLRPC_LIB_EXPORTED
 void 
 xmlrpc_parse_value_va(xmlrpc_env *   const envP,
                       xmlrpc_value * const value,
@@ -564,28 +716,33 @@ typedef enum xmlrpc_dialect {
     xmlrpc_dialect_apache
 } xmlrpc_dialect;
 
+XMLRPC_LIB_EXPORTED
 void 
 xmlrpc_serialize_value2(xmlrpc_env *       const envP,
                         xmlrpc_mem_block * const outputP,
                         xmlrpc_value *     const valueP,
                         xmlrpc_dialect     const dialect);
 
+XMLRPC_LIB_EXPORTED
 void
 xmlrpc_serialize_value(xmlrpc_env *       const envP,
                        xmlrpc_mem_block * const outputP,
                        xmlrpc_value *     const valueP);
 
+XMLRPC_LIB_EXPORTED
 void 
 xmlrpc_serialize_params2(xmlrpc_env *       const envP,
                          xmlrpc_mem_block * const outputP,
                          xmlrpc_value *     const paramArrayP,
                          xmlrpc_dialect     const dialect);
 
+XMLRPC_LIB_EXPORTED
 void
 xmlrpc_serialize_params(xmlrpc_env *       const envP,
                         xmlrpc_mem_block * const outputP,
                         xmlrpc_value *     const paramArrayP);
 
+XMLRPC_LIB_EXPORTED
 void 
 xmlrpc_serialize_call2(xmlrpc_env *       const envP,
                        xmlrpc_mem_block * const outputP,
@@ -593,23 +750,27 @@ xmlrpc_serialize_call2(xmlrpc_env *       const envP,
                        xmlrpc_value *     const paramArrayP,
                        xmlrpc_dialect     const dialect);
 
+XMLRPC_LIB_EXPORTED
 void 
 xmlrpc_serialize_call(xmlrpc_env *       const envP,
                       xmlrpc_mem_block * const outputP,
                       const char *       const methodName,
                       xmlrpc_value *     const paramArrayP);
 
+XMLRPC_LIB_EXPORTED
 void 
 xmlrpc_serialize_response2(xmlrpc_env *       const envP,
                            xmlrpc_mem_block * const outputP,
                            xmlrpc_value *     const valueP,
                            xmlrpc_dialect     const dialect);
 
+XMLRPC_LIB_EXPORTED
 void
 xmlrpc_serialize_response(xmlrpc_env *       const envP,
                           xmlrpc_mem_block * const outputP,
                           xmlrpc_value *     const valueP);
 
+XMLRPC_LIB_EXPORTED
 void
 xmlrpc_serialize_fault(xmlrpc_env *       const envP,
                        xmlrpc_mem_block * const outputP,
@@ -620,17 +781,22 @@ xmlrpc_serialize_fault(xmlrpc_env *       const envP,
 **  Decoding XML
 **=======================================================================*/
 
-/* Parse an XML-RPC call. If an error occurs, set a fault and set
-** the output variables to NULL.
-** The caller is responsible for calling free(*out_method_name) and
-** xmlrpc_DECREF(*out_param_array). */
+XMLRPC_LIB_EXPORTED
+void
+xmlrpc_parse_value_xml(xmlrpc_env *    const envP,
+                       const char *    const xmlData,
+                       size_t          const xmlDataLen,
+                       xmlrpc_value ** const valuePP);
+
+XMLRPC_LIB_EXPORTED
 void 
 xmlrpc_parse_call(xmlrpc_env *    const envP,
-                  const char *    const xml_data,
-                  size_t          const xml_len,
-                  const char **   const out_method_name,
-                  xmlrpc_value ** const out_param_array);
+                  const char *    const xmlData,
+                  size_t          const xmlDataLen,
+                  const char **   const methodNameP,
+                  xmlrpc_value ** const paramArrayPP);
 
+XMLRPC_LIB_EXPORTED
 void
 xmlrpc_parse_response2(xmlrpc_env *    const envP,
                        const char *    const xmlData,
@@ -642,40 +808,13 @@ xmlrpc_parse_response2(xmlrpc_env *    const envP,
 
 /* xmlrpc_parse_response() is for backward compatibility */
 
+XMLRPC_LIB_EXPORTED
 xmlrpc_value *
 xmlrpc_parse_response(xmlrpc_env * const envP, 
                       const char * const xmlData, 
                       size_t       const xmlDataLen);
 
 
-/*=========================================================================
-**  XML-RPC Base64 Utilities
-**=========================================================================
-**  Here are some lightweight utilities which can be used to encode and
-**  decode Base64 data. These are exported mainly for testing purposes.
-*/
-
-/* This routine inserts newlines every 76 characters, as required by the
-** Base64 specification. */
-xmlrpc_mem_block *
-xmlrpc_base64_encode(xmlrpc_env *    env,
-                     unsigned char * bin_data,
-                     size_t          bin_len);
-
-/* This routine encodes everything in one line. This is needed for HTTP
-** authentication and similar tasks. */
-xmlrpc_mem_block *
-xmlrpc_base64_encode_without_newlines(xmlrpc_env *    env,
-                                      unsigned char * bin_data,
-                                      size_t          bin_len);
-
-/* This decodes Base64 data with or without newlines. */
-extern xmlrpc_mem_block *
-xmlrpc_base64_decode(xmlrpc_env * const envP,
-                     const char * const ascii_data,
-                     size_t       const ascii_len);
-
-
 /*=========================================================================
 **  Authorization Cookie Handling
 **=========================================================================
@@ -686,10 +825,12 @@ xmlrpc_base64_decode(xmlrpc_env * const envP,
 **  a cookie replacement of basic authentication.)
 **/
 
+XMLRPC_LIB_EXPORTED
 extern void xmlrpc_authcookie_set(xmlrpc_env * const env,
                                   const char * const username,
                                   const char * const password);
 
+XMLRPC_LIB_EXPORTED
 char *xmlrpc_authcookie(void);
 
 /*=========================================================================
@@ -718,9 +859,11 @@ char *xmlrpc_authcookie(void);
 #define XMLRPC_XML_SIZE_LIMIT_DEFAULT (512*1024)
 
 /* Set a specific limit to the specified value. */
+XMLRPC_LIB_EXPORTED
 extern void xmlrpc_limit_set (int const limit_id, size_t const value);
 
 /* Get the value of a specified limit. */
+XMLRPC_LIB_EXPORTED
 extern size_t xmlrpc_limit_get (int const limit_id);
 
 
