@@ -2,6 +2,8 @@
     curlTransaction
 =============================================================================*/
 
+#define _XOPEN_SOURCE 600  /* Make sure strdup() is in <string.h> */
+
 #include <assert.h>
 #include <string.h>
 #include <stdlib.h>
@@ -15,7 +17,9 @@
 #include "version.h"
 
 #include <curl/curl.h>
+#ifdef NEED_CURL_TYPES_H
 #include <curl/types.h>
+#endif
 #include <curl/easy.h>
 
 #include "curlversion.h"
@@ -54,12 +58,21 @@ struct curlTransaction {
     struct curl_slist * headerList;
         /* The HTTP headers for the transaction */
     const char * serverUrl;  /* malloc'ed - belongs to this object */
+    xmlrpc_mem_block * postDataP;
+        /* The data to send for the POST method */
+    xmlrpc_mem_block * responseDataP;
+        /* This is normally where to put the body of the HTTP response.  But
+           do to a quirk of Curl, if the response is not valid HTTP, rather
+           than this just being irrelevant, it is the place that Curl puts the
+           server's non-HTTP response.  That can be useful for error
+           reporting.
+        */
 };
 
 
 
 static void
-addHeader(xmlrpc_env * const envP,
+addHeader(xmlrpc_env *         const envP,
           struct curl_slist ** const headerListP,
           const char *         const headerText) {
 
@@ -84,39 +97,82 @@ addContentTypeHeader(xmlrpc_env *         const envP,
 
 
 
+static const char *
+xmlrpcUserAgentPart(bool const reportIt) {
+
+    const char * retval;
+    
+    if (reportIt) {
+        curl_version_info_data * const curlInfoP =
+            curl_version_info(CURLVERSION_NOW);
+        char curlVersion[32];
+        
+        snprintf(curlVersion, sizeof(curlVersion), "%u.%u.%u",
+                 (curlInfoP->version_num >> 16) & 0xff,
+                 (curlInfoP->version_num >>  8) & 0xff,
+                 (curlInfoP->version_num >>  0) & 0xff
+            );
+
+        xmlrpc_asprintf(&retval,
+                        "Xmlrpc-c/%s Curl/%s",
+                        XMLRPC_C_VERSION, curlVersion);
+    } else
+        xmlrpc_asprintf(&retval, "%s", "");
+
+    return retval;
+}
+
+
+
 static void
 addUserAgentHeader(xmlrpc_env *         const envP,
                    struct curl_slist ** const headerListP,
+                   bool                 const reportXmlrpc,
                    const char *         const userAgent) {
+/*----------------------------------------------------------------------------
+   Add a User-Agent HTTP header to the Curl header list *headerListP,
+   if appropriate.
+   
+   'reportXmlrpc' means we want to tell the client what XML-RPC agent
+   is being used -- Xmlrpc-c and layers below.
+
+   'userAgent' is a string describing the layers above Xmlrpc-c.  We
+   assume it is in the proper format to be included in a User-Agent
+   header.  (We should probably fix that some day -- take ownership
+   of that format).
+-----------------------------------------------------------------------------*/
+    if (reportXmlrpc || userAgent) {
+        /* Add the header */
 
-    if (userAgent) {
         /* Note: Curl has a CURLOPT_USERAGENT option that does some of this
            work.  We prefer to be totally in control, though, so we build
            the header explicitly.
         */
-    
-        curl_version_info_data * const curlInfoP =
-            curl_version_info(CURLVERSION_NOW);
-        char curlVersion[32];
-        const char * userAgentHeader;
-        
-        snprintf(curlVersion, sizeof(curlVersion), "%u.%u.%u",
-                (curlInfoP->version_num >> 16) && 0xff,
-                (curlInfoP->version_num >>  8) && 0xff,
-                (curlInfoP->version_num >>  0) && 0xff
-            );
-                  
-        xmlrpc_asprintf(&userAgentHeader,
-                        "User-Agent: %s Xmlrpc-c/%s Curl/%s",
-                        userAgent, XMLRPC_C_VERSION, curlVersion);
-        
-        if (userAgentHeader == xmlrpc_strsol)
+
+        const char * const xmlrpcPart = xmlrpcUserAgentPart(reportXmlrpc);
+
+        if (xmlrpc_strnomem(xmlrpcPart))
             xmlrpc_faultf(envP, "Couldn't allocate memory for "
                           "User-Agent header");
         else {
-            addHeader(envP, headerListP, userAgentHeader);
+            const char * const userPart = userAgent ? userAgent : "";
+            const char * const space = userAgent && reportXmlrpc ? " " : "";
+
+            const char * userAgentHeader;
+
+            xmlrpc_asprintf(&userAgentHeader,
+                            "User-Agent: %s%s%s",
+                            userPart, space, xmlrpcPart);
+        
+            if (xmlrpc_strnomem(userAgentHeader))
+                xmlrpc_faultf(envP, "Couldn't allocate memory for "
+                              "User-Agent header");
+            else {
+                addHeader(envP, headerListP, userAgentHeader);
             
-            xmlrpc_strfree(userAgentHeader);
+                xmlrpc_strfree(userAgentHeader);
+            }
+            xmlrpc_strfree(xmlrpcPart);
         }
     }
 }
@@ -132,7 +188,7 @@ addAuthorizationHeader(xmlrpc_env *         const envP,
             
     xmlrpc_asprintf(&authorizationHeader, "Authorization: %s", hdrValue);
     
-    if (authorizationHeader == xmlrpc_strsol)
+    if (xmlrpc_strnomem(authorizationHeader))
         xmlrpc_faultf(envP, "Couldn't allocate memory for "
                       "Authorization header");
     else {
@@ -144,9 +200,52 @@ addAuthorizationHeader(xmlrpc_env *         const envP,
 
 
 
+/*
+  In HTTP 1.1, the client can send the header "Expect: 100-continue", which
+  tells the server that the client isn't going to send the body until the
+  server tells it to by sending a "continue" response (HTTP response code 100).
+  The server is obligated to send that response.
+
+  However, many servers are broken and don't send the Continue response.
+
+  Early libcurl did not send the Expect: header, thus worked fine with such
+  broken servers.  But as of ca. 2007, libcurl sends the Expect:, and waits
+  for the response, when the body is large.  It gives up after 3 seconds and
+  sends the body anyway.
+
+  To accomodate the broken servers and for backward compatibility, we always
+  force libcurl not to send the Expect and consequently not to wait for the
+  response, using the hackish (but according to libcurl design) method of
+  including an entry in our explicit header list that is an Expect: header
+  with an empty argument.  This causes libcurl not to send any Expect: header.
+  This is since 1.19; we may find there are also servers and/or libcurl levels
+  that can't work with that.
+
+  We may find a case where the Expect/Continue protocol is desirable.  If we
+  do, we should add a transport option to request the function and let libcurl
+  do its thing when the user requests it.
+
+  The purpose of Expect/Continue is to save the client the trouble of
+  generating and/or sending the body when the server is just going to reject
+  the transaction based on the headers -- like maybe because the body is
+  too big.
+*/
+
+
+static void
+addExpectHeader(xmlrpc_env *         const envP,
+                struct curl_slist ** const headerListP) {
+
+    addHeader(envP, headerListP, "Expect:");
+        /* Don't send Expect header.  See explanation above. */
+}
+
+
+
 static void
 createCurlHeaderList(xmlrpc_env *               const envP,
                      const char *               const authHdrValue,
+                     bool                       const dontAdvertise,
                      const char *               const userAgent,
                      struct curl_slist **       const headerListP) {
 
@@ -156,16 +255,18 @@ createCurlHeaderList(xmlrpc_env *               const envP,
 
     addContentTypeHeader(envP, &headerList);
     if (!envP->fault_occurred) {
-        addUserAgentHeader(envP, &headerList, userAgent);
+        addUserAgentHeader(envP, &headerList, !dontAdvertise, userAgent);
         if (!envP->fault_occurred) {
             if (authHdrValue)
                 addAuthorizationHeader(envP, &headerList, authHdrValue);
         }
+        if (!envP->fault_occurred)
+            addExpectHeader(envP, &headerList);
     }
     if (envP->fault_occurred)
         curl_slist_free_all(headerList);
-    else
-        *headerListP = headerList;
+
+    *headerListP = headerList;
 }
 
 
@@ -174,13 +275,16 @@ static size_t
 collect(void *  const ptr, 
         size_t  const size, 
         size_t  const nmemb,  
-        FILE  * const stream) {
+        void  * const streamP) {
 /*----------------------------------------------------------------------------
-   This is a Curl output function.  Curl calls this to deliver the
-   HTTP response body to the Curl client.  Curl thinks it's writing to
-   a POSIX stream.
+   This is a Curl write function.  Curl calls this to deliver the
+   HTTP response body to the Curl client.
+
+   But as a design quirk, Curl also calls this when there is no HTTP body
+   because the response from the server is not valid HTTP.  In that case,
+   Curl calls this to deliver the raw contents of the response.
 -----------------------------------------------------------------------------*/
-    xmlrpc_mem_block * const responseXmlP = (xmlrpc_mem_block *) stream;
+    xmlrpc_mem_block * const responseXmlP = streamP;
     char * const buffer = ptr;
     size_t const length = nmemb * size;
 
@@ -202,42 +306,40 @@ collect(void *  const ptr,
 
 static int
 curlProgress(void * const contextP,
-             double const dltotal  ATTR_UNUSED,
-             double const dlnow    ATTR_UNUSED,
-             double const ultotal  ATTR_UNUSED,
-             double const ulnow    ATTR_UNUSED) {
+             double const dltotal,
+             double const dlnow,
+             double const ultotal,
+             double const ulnow) {
 /*----------------------------------------------------------------------------
-   This is a Curl "progress function."  It's something various Curl
-   functions call every so often, including whenever something gets
-   interrupted by the process receiving, and catching, a signal.
-   There are two purposes of a Curl progress function: 1) lets us log
-   the progress of a long-running transaction such as a big download,
-   e.g. by displaying a progress bar somewhere.  In Xmlrpc-c, we don't
-   implement this purpose.  2) allows us to tell the Curl function,
-   via our return code, that calls it that we don't want to wait
-   anymore for the operation to complete.
+   This is a Curl "progress function."  It's something various Curl functions
+   call every so often, including whenever something gets interrupted by the
+   process receiving, and catching, a signal.  There are two purposes of a
+   Curl progress function: 1) lets us log the progress of a long-running
+   transaction such as a big download, e.g. by displaying a progress bar
+   somewhere.  2) allows us to tell the Curl function, via our return code,
+   that calls it that we don't want to wait anymore for the operation to
+   complete.
 
-   In Curl versions before March 2007, we get called once per second
-   and signals have no effect.  In current Curl, we usually get called
-   immediately after a signal gets caught while Curl is waiting to
-   receive a response from the server.  But Curl doesn't properly
-   synchronize with signals, so it may miss one and then we don't get
-   called until the next scheduled one-per-second call.
+   In Curl versions before March 2007, we get called once per second and
+   signals have no effect.  In current Curl, we usually get called immediately
+   after a signal gets caught while Curl is waiting to receive a response from
+   the server.  But Curl doesn't properly synchronize with signals, so it may
+   miss one and then we don't get called until the next scheduled
+   one-per-second call.
 
-   All we do is tell Caller it's time to give up if the transport's
-   client says it is via his "interrupt" flag.
+   All we do is pass the call through to the curlTransaction's progress
+   function (the one that the creator of the curlTransaction registered).
 
-   This function is not as important as it once was.  This module used
-   to use curl_easy_perform(), which can be interrupted only via this
-   progress function.  But because of the above-mentioned failure of
-   Curl to properly synchronize signals (and Bryan's failure to get
-   Curl developers to accept code to fix it), we now use the Curl
-   "multi" facility instead and do our own pselect().  But
-   This function still normally gets called by curl_multi_perform(),
-   which the transport tries to call even when the user has requested
-   interruption, because we don't trust our ability to abort a running
-   Curl transaction.  curl_multi_perform() reliably winds up a Curl
-   transaction when this function tells it to.
+   This function is not as important as it once was for interrupting purposes.
+   This module used to use curl_easy_perform(), which can be interrupted only
+   via this progress function.  But because of the above-mentioned failure of
+   Curl to properly synchronize signals (and Bryan's failure to get Curl
+   developers to accept code to fix it), we now use the Curl "multi" facility
+   instead and do our own pselect().  But this function still normally gets
+   called by curl_multi_perform(), which the transport tries to call even when
+   the user has requested interruption, because we don't trust our ability to
+   abort a running Curl transaction.  curl_multi_perform() reliably winds up a
+   Curl transaction when this function tells it to.
 -----------------------------------------------------------------------------*/
     curlTransaction * const curlTransactionP = contextP;
 
@@ -249,7 +351,9 @@ curlProgress(void * const contextP,
     assert(curlTransactionP);
     assert(curlTransactionP->progress);
 
-    curlTransactionP->progress(curlTransactionP->userContextP, &abort);
+    curlTransactionP->progress(curlTransactionP->userContextP,
+                               dltotal, dlnow, ultotal, ulnow,
+                               &abort);
 
     return abort;
 }
@@ -272,50 +376,50 @@ setupAuth(xmlrpc_env *               const envP ATTR_UNUSED,
    HTTP basic authentication.
 
    So the special function is this: if libcurl is too old to have
-   authorization options and *serverInfoP allows basic authentication,
-   return as *basicAuthHdrParamP an appropriate parameter for the
-   Authorization: Basic: HTTP header.  Otherwise, return
-   *basicAuthHdrParamP == NULL.
+   authorization options and *serverInfoP allows basic authentication, return
+   as *authHdrValueP an appropriate parameter for the Authorization: Basic:
+   HTTP header.  Otherwise, return *authHdrValueP == NULL.
 -----------------------------------------------------------------------------*/
-    if (serverInfoP->allowedAuth.basic) {
-        CURLcode rc;
-        rc = curl_easy_setopt(curlSessionP, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
+    CURLcode rc;
 
-        if (rc == CURLE_OK)
-            *authHdrValueP = NULL;
-        else {
-            *authHdrValueP = strdup(serverInfoP->basicAuthHdrValue);
-            if (*authHdrValueP == NULL)
-                xmlrpc_faultf(envP, "Unable to allocate memory for basic "
-                              "authentication header");
-        }
-    } else
-        *authHdrValueP = NULL;
+    /* We don't worry if libcurl is too old for specific kinds of
+       authentication; they're defined only as _allowed_ authentication
+       methods, for when client and server are capable of using it, and unlike
+       with basic authentication, we have no historical commitment to consider
+       an old libcurl as capable of doing these.
 
-    /* We don't worry if libcurl is too old for these other kinds of
-       authentication; they're only defined as _allowed_
-       authentication methods, for when client and server are capable
-       of using it, and unlike with basic authentication, we have no
-       historical commitment to consider an old libcurl as capable of
-       doing these.
+       Note that curl_easy_setopt(CURLOPT_HTTPAUTH) succeeds even if there are
+       flags in it argument that weren't defined when it was written.
     */
     
     if (serverInfoP->userNamePw)
         curl_easy_setopt(curlSessionP, CURLOPT_USERPWD,
                          serverInfoP->userNamePw);
 
-    if (serverInfoP->allowedAuth.digest)
-        curl_easy_setopt(
-            curlSessionP, CURLOPT_HTTPAUTH, CURLAUTH_DIGEST);
-    if (serverInfoP->allowedAuth.gssnegotiate)
-        curl_easy_setopt(
-            curlSessionP, CURLOPT_HTTPAUTH, CURLAUTH_GSSNEGOTIATE);
-    if (serverInfoP->allowedAuth.ntlm)
-        curl_easy_setopt(
-            curlSessionP, CURLOPT_HTTPAUTH, CURLAUTH_NTLM);
+    rc = curl_easy_setopt(
+        curlSessionP, CURLOPT_HTTPAUTH,
+        (serverInfoP->allowedAuth.basic        ? CURLAUTH_BASIC        : 0) |
+        (serverInfoP->allowedAuth.digest       ? CURLAUTH_DIGEST       : 0) |
+        (serverInfoP->allowedAuth.gssnegotiate ? CURLAUTH_GSSNEGOTIATE : 0) |
+        (serverInfoP->allowedAuth.ntlm         ? CURLAUTH_NTLM         : 0));
+    
+    if (rc != CURLE_OK) {
+        /* Curl is too old to do authentication, so we do it ourselves
+           with an explicit header if we have to.
+        */
+        if (serverInfoP->allowedAuth.basic) {
+            *authHdrValueP = strdup(serverInfoP->basicAuthHdrValue);
+            if (*authHdrValueP == NULL)
+                xmlrpc_faultf(envP, "Unable to allocate memory for basic "
+                              "authentication header");
+        } else        
+            *authHdrValueP = NULL;
+    } else
+        *authHdrValueP = NULL;
 }
 
 
+
 static void
 setCurlTimeout(CURL *       const curlSessionP ATTR_UNUSED,
                unsigned int const timeout ATTR_UNUSED) {
@@ -344,23 +448,123 @@ assertConstantsMatch(void) {
    formally.
 -----------------------------------------------------------------------------*/
     assert(XMLRPC_SSLVERSION_DEFAULT == CURL_SSLVERSION_DEFAULT);
-    assert(XMLRPC_SSLVERSION_TLSv1   == CURL_SSLVERSION_TLSv1);
-    assert(XMLRPC_SSLVERSION_SSLv2   == CURL_SSLVERSION_SSLv2);
-    assert(XMLRPC_SSLVERSION_SSLv3   == CURL_SSLVERSION_SSLv3);
+    assert(XMLRPC_SSLVERSION_TLSv1   == CURL_SSLVERSION_TLSv1  );
+    assert(XMLRPC_SSLVERSION_SSLv2   == CURL_SSLVERSION_SSLv2  );
+    assert(XMLRPC_SSLVERSION_SSLv3   == CURL_SSLVERSION_SSLv3  );
+
+    assert(XMLRPC_HTTPAUTH_BASIC        == CURLAUTH_BASIC       );
+    assert(XMLRPC_HTTPAUTH_DIGEST       == CURLAUTH_DIGEST      );
+    assert(XMLRPC_HTTPAUTH_GSSNEGOTIATE == CURLAUTH_GSSNEGOTIATE);
+    assert(XMLRPC_HTTPAUTH_NTLM         == CURLAUTH_NTLM        );
+
+    assert(XMLRPC_HTTPPROXY_HTTP   == CURLPROXY_HTTP   );
+    assert(XMLRPC_HTTPPROXY_SOCKS5 == CURLPROXY_SOCKS5 );
+}
+
+
+
+/* About Curl and GSSAPI credential delegation:
+
+   Up through Curl 7.21.6, libcurl always delegates GSSAPI credentials, which
+   means it gives the client's secrets to the server so the server can operate
+   on the client's behalf.  In mid-2011, this was noticed to be a major
+   security exposure, because the server is not necessarily trustworthy.
+   One is supposed to delegate one's credentials only to a server one trusts.
+   So in 7.21.7, Curl never delegates GSSAPI credentials.
+
+   But that causes problems for clients that _do_ trust their server, which
+   had always relied upon Curl's delegation.
+
+   So starting in 7.22.0, Curl gives the user the choice.  The default is no
+   delegation, but the Curl user can set the CURLOPT_GSSAPI_DELEGATION flag to
+   order delegation.
+
+   Complicating matters is that some people made local variations of Curl
+   during the transition phase, so the version number alone isn't
+   determinative, so we rely on it only where we have to.
+
+   So Xmlrpc-c gives the same choice to its own user, via its
+   'gssapi_delegation' Curl transport option.
+   
+   Current Xmlrpc-c can be linked with, and compiled with, any version of
+   Curl, so it has to carefully consider all the possibilities.
+*/
+
+
+
+static bool
+curlAlwaysDelegatesGssapi(void) {
+/*----------------------------------------------------------------------------
+   The Curl library we're using always delegates GSSAPI credentials
+   (we don't have a choice).
+
+   This works with Curl as distributed by the Curl project, but there are
+   other versions of Curl for which it doesn't -- those versions report
+   older version numbers but in fact don't always delegate.  Some never
+   delegate, and some give the user the option.
+-----------------------------------------------------------------------------*/
+    curl_version_info_data * const curlInfoP =
+        curl_version_info(CURLVERSION_NOW);
+
+    return (curlInfoP->version_num <= 0x071506);  /* 7.21.6 */
+}
+
+
+
+static void
+requestGssapiDelegation(CURL * const curlSessionP ATTR_UNUSED,
+                        bool * const gotItP) {
+/*----------------------------------------------------------------------------
+   Set up the Curl session *curlSessionP to delegate its GSSAPI credentials to
+   the server.
+
+   Return *gotitP is true iff we succeed.  We fail when the version of libcurl
+   for which we are compiled or to which we are linked is not capable of such
+   delegation.
+-----------------------------------------------------------------------------*/
+#if HAVE_CURL_GSSAPI_DELEGATION
+    int rc;
+
+    rc = curl_easy_setopt(curlSessionP, CURLOPT_GSSAPI_DELEGATION,
+                          CURLGSSAPI_DELEGATION_FLAG);
+
+    if (rc == CURLE_OK)
+        *gotItP = true;
+    else {
+        /* The only way curl_easy_setopt() could have failed is that we
+           are running with an old libcurl from before
+           CURLOPT_GSSAPI_DELEGATION was invented.
+        */
+        if (curlAlwaysDelegatesGssapi()) {
+            /* No need to request delegation; we got it anyway */
+            *gotItP = true;
+        } else
+            *gotItP = false;
+    }
+#else
+    if (curlAlwaysDelegatesGssapi())
+        *gotItP = true;
+    else {
+        /* The library may be able to do credential delegation on request, but
+           we have no way to request it; the Curl for which we are compiled is
+           too old.
+        */
+        *gotItP = false;
+    }
+#endif
 }
 
 
 
 static void
 setupCurlSession(xmlrpc_env *               const envP,
-                 curlTransaction *          const curlTransactionP,
-                 xmlrpc_mem_block *         const callXmlP,
-                 xmlrpc_mem_block *         const responseXmlP,
+                 curlTransaction *          const transP,
                  const xmlrpc_server_info * const serverInfoP,
+                 bool                       const dontAdvertise,
                  const char *               const userAgent,
                  const struct curlSetup *   const curlSetupP) {
 /*----------------------------------------------------------------------------
-   Set up the Curl session for the transaction *curlTransactionP so that
+   Set up the Curl session for the transaction *transP so that
    a subsequent curl_easy_perform() would perform said transaction.
 
    The data curl_easy_perform() would send for that transaction would 
@@ -372,7 +576,7 @@ setupCurlSession(xmlrpc_env *               const envP,
    Xmlrpc-c interface.  Some day, we need to replace this with a type
    (probably identical) not tied to Xmlrpc-c.
 -----------------------------------------------------------------------------*/
-    CURL * const curlSessionP = curlTransactionP->curlSessionP;
+    CURL * const curlSessionP = transP->curlSessionP;
 
     assertConstantsMatch();
 
@@ -383,26 +587,32 @@ setupCurlSession(xmlrpc_env *               const envP,
        tell us that something finished on a particular session, not that
        a particular transaction finished.
     */
-    curl_easy_setopt(curlSessionP, CURLOPT_PRIVATE, curlTransactionP);
+
+    /* It is our policy to do a libcurl call only where necessary, I.e.  not
+       to set what is the default anyhow.  The reduction in calls may save
+       some time, but mostly, it will save us encountering rare bugs or
+       suffering from backward incompatibilities in future libcurl.  I.e. we
+       don't exercise any more of libcurl than we have to.
+    */
+
+    curl_easy_setopt(curlSessionP, CURLOPT_PRIVATE, transP);
 
     curl_easy_setopt(curlSessionP, CURLOPT_POST, 1);
-    curl_easy_setopt(curlSessionP, CURLOPT_URL, curlTransactionP->serverUrl);
+    curl_easy_setopt(curlSessionP, CURLOPT_URL, transP->serverUrl);
 
-    XMLRPC_MEMBLOCK_APPEND(char, envP, callXmlP, "\0", 1);
+    XMLRPC_MEMBLOCK_APPEND(char, envP, transP->postDataP, "\0", 1);
     if (!envP->fault_occurred) {
         curl_easy_setopt(curlSessionP, CURLOPT_POSTFIELDS, 
-                         XMLRPC_MEMBLOCK_CONTENTS(char, callXmlP));
+                         XMLRPC_MEMBLOCK_CONTENTS(char, transP->postDataP));
         curl_easy_setopt(curlSessionP, CURLOPT_WRITEFUNCTION, collect);
-        curl_easy_setopt(curlSessionP, CURLOPT_FILE, responseXmlP);
+        curl_easy_setopt(curlSessionP, CURLOPT_FILE, transP->responseDataP);
         curl_easy_setopt(curlSessionP, CURLOPT_HEADER, 0);
-        curl_easy_setopt(curlSessionP, CURLOPT_ERRORBUFFER, 
-                         curlTransactionP->curlError);
-        if (curlTransactionP->progress) {
+        curl_easy_setopt(curlSessionP, CURLOPT_ERRORBUFFER, transP->curlError);
+        if (transP->progress) {
             curl_easy_setopt(curlSessionP, CURLOPT_NOPROGRESS, 0);
             curl_easy_setopt(curlSessionP, CURLOPT_PROGRESSFUNCTION,
                              curlProgress);
-            curl_easy_setopt(curlSessionP, CURLOPT_PROGRESSDATA,
-                             curlTransactionP);
+            curl_easy_setopt(curlSessionP, CURLOPT_PROGRESSDATA, transP);
         } else
             curl_easy_setopt(curlSessionP, CURLOPT_NOPROGRESS, 1);
         
@@ -414,6 +624,9 @@ setupCurlSession(xmlrpc_env *               const envP,
         if (curlSetupP->networkInterface)
             curl_easy_setopt(curlSessionP, CURLOPT_INTERFACE,
                              curlSetupP->networkInterface);
+        if (curlSetupP->referer)
+            curl_easy_setopt(curlSessionP, CURLOPT_REFERER,
+                             curlSetupP->referer);
         if (curlSetupP->sslCert)
             curl_easy_setopt(curlSessionP, CURLOPT_SSLCERT,
                              curlSetupP->sslCert);
@@ -457,10 +670,44 @@ setupCurlSession(xmlrpc_env *               const envP,
             curl_easy_setopt(curlSessionP, CURLOPT_SSL_CIPHER_LIST,
                              curlSetupP->sslCipherList);
 
+        if (curlSetupP->proxy)
+            curl_easy_setopt(curlSessionP, CURLOPT_PROXY, curlSetupP->proxy);
+        if (curlSetupP->proxyAuth != CURLAUTH_BASIC)
+            /* Note that the Xmlrpc-c default and the Curl default are
+               different.  Xmlrpc-c is none, while Curl is basic.  One reason
+               for this is that it makes our extensible parameter list scheme,
+               wherein zero always means default, easier.
+            */
+            curl_easy_setopt(curlSessionP, CURLOPT_PROXYAUTH,
+                             curlSetupP->proxyAuth);
+        if (curlSetupP->proxyPort)
+            curl_easy_setopt(curlSessionP, CURLOPT_PROXYPORT,
+                             curlSetupP->proxyPort);
+        if (curlSetupP->proxyUserPwd)
+            curl_easy_setopt(curlSessionP, CURLOPT_PROXYUSERPWD,
+                             curlSetupP->proxyUserPwd);
+        if (curlSetupP->proxyType)
+            curl_easy_setopt(curlSessionP, CURLOPT_PROXYTYPE,
+                             curlSetupP->proxyType);
+
+        if (curlSetupP->verbose)
+            curl_easy_setopt(curlSessionP, CURLOPT_VERBOSE, 1l);
+
         if (curlSetupP->timeout)
             setCurlTimeout(curlSessionP, curlSetupP->timeout);
 
-        {
+        if (curlSetupP->gssapiDelegation) {
+            bool gotIt;
+            requestGssapiDelegation(curlSessionP, &gotIt);
+
+            if (!gotIt)
+                xmlrpc_faultf(envP, "Cannot honor 'gssapi_delegation' "
+                              "Curl transport option.  "
+                              "This version of libcurl is not "
+                              "capable of delegating GSSAPI credentials");
+        }
+
+        if (!envP->fault_occurred) {
             const char * authHdrValue;
                 /* NULL means we don't have to construct an explicit
                    Authorization: header.  non-null means we have to
@@ -470,12 +717,13 @@ setupCurlSession(xmlrpc_env *               const envP,
             setupAuth(envP, curlSessionP, serverInfoP, &authHdrValue);
             if (!envP->fault_occurred) {
                 struct curl_slist * headerList;
-                createCurlHeaderList(envP, authHdrValue, userAgent,
+                createCurlHeaderList(envP, authHdrValue,
+                                     dontAdvertise, userAgent,
                                      &headerList);
                 if (!envP->fault_occurred) {
                     curl_easy_setopt(
                         curlSessionP, CURLOPT_HTTPHEADER, headerList);
-                    curlTransactionP->headerList = headerList;
+                    transP->headerList = headerList;
                 }
                 if (authHdrValue)
                     xmlrpc_strfree(authHdrValue);
@@ -492,6 +740,7 @@ curlTransaction_create(xmlrpc_env *               const envP,
                        const xmlrpc_server_info * const serverP,
                        xmlrpc_mem_block *         const callXmlP,
                        xmlrpc_mem_block *         const responseXmlP,
+                       bool                       const dontAdvertise,
                        const char *               const userAgent,
                        const struct curlSetup *   const curlSetupStuffP,
                        void *                     const userContextP,
@@ -514,9 +763,11 @@ curlTransaction_create(xmlrpc_env *               const envP,
         if (curlTransactionP->serverUrl == NULL)
             xmlrpc_faultf(envP, "Out of memory to store server URL.");
         else {
+            curlTransactionP->postDataP     = callXmlP;
+            curlTransactionP->responseDataP = responseXmlP;
+
             setupCurlSession(envP, curlTransactionP,
-                             callXmlP, responseXmlP,
-                             serverP, userAgent,
+                             serverP, dontAdvertise, userAgent,
                              curlSetupStuffP);
             
             if (envP->fault_occurred)
@@ -554,10 +805,55 @@ interpretCurlEasyError(const char ** const descriptionP,
 
 
 
+/* CURL quirks:
+
+   We have seen Curl report that the transaction completed OK (CURLE_OK) when
+   the server sent back garbage instead of an HTTP response (because it wasn't
+   an HTTP server).  In that case Curl reports zero in place of the response
+   code.  It's strange that Curl doesn't report that protocol violation at a
+   higher level (perhaps with more detail), but apparently it does not, so we
+   go by the HTTP_CODE value.  Note that if the server closes the connection
+   without responding at all, Curl calls the transaction failed with an "empty
+   reply from server" error code.
+
+   It appears to be the case that when the server sends non-HTTP garbage, Curl
+   reports it as the HTTP response body.  E.g. we had an inetd server respond
+   with a "library not found" error message because the server connected
+   Standard Error to the socket.  The 'curl' program typed out the error
+   message, naked, and exited with exit status zero.  We exploit this
+   discovery to give better error reporting to our user.
+
+   We saw this with Curl 7.16.1.
+*/
+
+
+
+static const char *
+formatDataReceived(curlTransaction * const curlTransactionP) {
+
+    const char * retval;
+
+    if (XMLRPC_MEMBLOCK_SIZE(char, curlTransactionP->responseDataP) == 0)
+        retval = xmlrpc_strdupsol("");
+    else {
+        xmlrpc_asprintf(
+            &retval,
+            "Raw data from server: '%s'\n",
+            XMLRPC_MEMBLOCK_CONTENTS(char, curlTransactionP->responseDataP));
+    }
+    return retval;
+}
+
+
+
 void
 curlTransaction_getError(curlTransaction * const curlTransactionP,
                          xmlrpc_env *      const envP) {
-
+/*----------------------------------------------------------------------------
+   Determine whether the transaction *curlTransactionP was successful in HTTP
+   terms.  Assume the transaction did complete.  Return as *envP an indication
+   of whether the transaction failed and if so, how.
+-----------------------------------------------------------------------------*/
     if (curlTransactionP->result != CURLE_OK) {
         /* We've seen Curl just return a null string for an explanation
            (e.g. when TCP connect() fails because IP address doesn't exist).
@@ -589,7 +885,18 @@ curlTransaction_getError(curlTransaction * const curlTransactionP,
                 "curl_easy_getinfo(CURLINFO_HTTP_CODE) says: %s", 
                 curlTransactionP->curlError);
         else {
-            if (http_result != 200)
+            if (http_result == 0) {
+                /* See above for what this case means */
+                const char * const dataReceived =
+                    formatDataReceived(curlTransactionP);
+
+                xmlrpc_env_set_fault_formatted(
+                    envP, XMLRPC_NETWORK_ERROR,
+                    "Server is not an XML-RPC server.  Its response to our "
+                    "call is not valid HTTP.  Or it's valid HTTP with a "
+                    "response code of zero.  %s", dataReceived);
+                xmlrpc_strfree(dataReceived);
+            } else if (http_result != 200)
                 xmlrpc_env_set_fault_formatted(
                     envP, XMLRPC_NETWORK_ERROR,
                     "HTTP response code is %ld, not 200",
