@@ -128,7 +128,7 @@ carriageParmPtr::operator->() const {
 
 carriageParm *
 carriageParmPtr::get() const {
-    return dynamic_cast<carriageParm *>(objectP);
+    return dynamic_cast<carriageParm *>(this->objectP);
 }
 
 
@@ -372,6 +372,15 @@ xmlTransaction::finishErr(error const&) const {
 
 
 
+void
+xmlTransaction::progress(struct xmlrpc_progress_data const&) const {
+
+    // This is just the base class method.  A derived class may override
+    // this with something substantial.
+}
+
+
+
 xmlTransactionPtr::xmlTransactionPtr() {}
 
 
@@ -485,7 +494,7 @@ clientXmlTransport::asyncComplete(
                 XMLRPC_MEMBLOCK_SIZE(char, responseXmlMP));
             xmlTranCtlP->xmlTranP->finish(responseXml);
         }
-    } catch(error) {
+    } catch(exception const&) {
         /* We can't throw an error back to C code, and the async_complete
            interface does not provide for failure, so we define ->finish()
            as not being capable of throwing an error.
@@ -503,6 +512,18 @@ clientXmlTransport::asyncComplete(
 
 
 
+void
+clientXmlTransport::progress(
+    struct xmlrpc_call_info *   const callInfoP,
+    struct xmlrpc_progress_data const progressData) {
+
+    xmlTranCtl * const xmlTranCtlP = reinterpret_cast<xmlTranCtl *>(callInfoP);
+
+    xmlTranCtlP->xmlTranP->progress(progressData);
+}
+
+
+
 void
 clientXmlTransport::setInterrupt(int *) {
 
@@ -601,7 +622,7 @@ clientXmlTransport_http::start(
             this->c_transportP,
             carriageParmHttpP->c_serverInfoP,
             tranCtlP->callXmlP,
-            &this->asyncComplete,
+            &this->asyncComplete, &this->progress,
             reinterpret_cast<xmlrpc_call_info *>(tranCtlP));
 
         throwIfError(env);
@@ -852,17 +873,17 @@ client_xml::call(carriageParm * const  carriageParmP,
 
     try {
         this->implP->transportP->call(carriageParmP, callXml, &responseXml);
-    } catch (error const& error) {
+    } catch (exception const& e) {
         throwf("Unable to transport XML to server and "
-               "get XML response back.  %s", error.what());
+               "get XML response back.  %s", e.what());
     }
     xml::trace("XML-RPC RESPONSE", responseXml);
         
     try {
         xml::parseResponse(responseXml, outcomeP);
-    } catch (error const& error) {
+    } catch (exception const& e) {
         throwf("Response XML from server is not valid XML-RPC response.  %s",
-               error.what());
+               e.what());
     }
 }
  
@@ -1096,8 +1117,20 @@ rpc::notifyComplete() {
 
 }
 
+
+
+void
+rpc::progress(struct xmlrpc_progress_data const&) const {
+/*----------------------------------------------------------------------------
+  If the user is interested in tracking the progress of the RPC, he will
+  derive a class from xmlrpc_c::rpc and override this with a progress()
+  that does something, such as display a progress bar.
+-----------------------------------------------------------------------------*/
+
+}
     
 
+
 value
 rpc::getResult() const {
 
@@ -1215,6 +1248,15 @@ xmlTransaction_client::finishErr(error const& error) const {
 
 
 
+void
+xmlTransaction_client::progress(
+    struct xmlrpc_progress_data const& progressData) const {
+
+    this->tranP->progress(progressData);
+}
+
+
+
 xmlTransaction_clientPtr::xmlTransaction_clientPtr() {}
 
 
