@@ -25,18 +25,18 @@
   class members had to be declared public so that other components of
   the library could see them, but the user is not supposed to access
   those members.
-
 *****************************************************************************/
 
 #include <cstdlib>
 #include <string>
 #include <vector>
-#include <time.h>
+#include <ctime>
 
 #include "xmlrpc-c/girerr.hpp"
 using girerr::error;
 #include "xmlrpc-c/base.h"
 #include "xmlrpc-c/base_int.h"
+#include "xmlrpc-c/string_int.h"
 #include "env_wrap.hpp"
 
 #include "xmlrpc-c/base.hpp"
@@ -59,12 +59,34 @@ class cDatetimeValueWrapper {
 public:
     xmlrpc_value * valueP;
     
+    cDatetimeValueWrapper(xmlrpc_datetime const cppvalue) {
+        env_wrap env;
+        
+        this->valueP = xmlrpc_datetime_new(&env.env_c, cppvalue);
+        throwIfError(env);
+    }
     cDatetimeValueWrapper(time_t const cppvalue) {
         env_wrap env;
         
         this->valueP = xmlrpc_datetime_new_sec(&env.env_c, cppvalue);
         throwIfError(env);
     }
+#if XMLRPC_HAVE_TIMEVAL
+    cDatetimeValueWrapper(struct timeval const cppvalue) {
+        env_wrap env;
+        
+        this->valueP = xmlrpc_datetime_new_timeval(&env.env_c, cppvalue);
+        throwIfError(env);
+    }
+#endif
+#if XMLRPC_HAVE_TIMESPEC
+    cDatetimeValueWrapper(struct timespec const cppvalue) {
+        env_wrap env;
+        
+        this->valueP = xmlrpc_datetime_new_timespec(&env.env_c, cppvalue);
+        throwIfError(env);
+    }
+#endif
     ~cDatetimeValueWrapper() {
         xmlrpc_DECREF(this->valueP);
     }
@@ -94,13 +116,13 @@ public:
 
 namespace xmlrpc_c {
 
-value::value() {
+value::value() {   // default constructor
     this->cValueP = NULL;
 }
 
 
 
-value::value(xmlrpc_value * const valueP) {  // default constructor
+value::value(xmlrpc_value * const valueP) {
 
     this->instantiate(valueP);
 }
@@ -133,6 +155,31 @@ value::~value() {
 
 
 
+bool
+value::isInstantiated() const {
+/*----------------------------------------------------------------------------
+   Return whether the object is actually a value, as opposed to a placeholder
+   variable waiting to be assigned a value.
+-----------------------------------------------------------------------------*/
+    return (this->cValueP != NULL);
+}
+
+
+
+void
+value::validateInstantiated() const {    // private
+/*----------------------------------------------------------------------------
+   Throw an exception if the object is just a placeholder, rather than an
+   actual XML-RPC value.
+-----------------------------------------------------------------------------*/
+    if (!this->cValueP)
+        throw(error("Reference to xmlrpc_c::value that has not been "
+                    "instantiated.  (xmlrpc_c::value::isInstantiated may be "
+                    "useful in diagnosing)"));
+}
+
+
+
 void
 value::instantiate(xmlrpc_value * const valueP) {
 
@@ -158,6 +205,8 @@ value::appendToCArray(xmlrpc_value * const arrayP) const {
 /*----------------------------------------------------------------------------
   Append this value to the C array 'arrayP'.
 ----------------------------------------------------------------------------*/
+    this->validateInstantiated();
+
     env_wrap env;
 
     xmlrpc_array_append_item(&env.env_c, arrayP, this->cValueP);
@@ -173,6 +222,8 @@ value::addToCStruct(xmlrpc_value * const structP,
 /*----------------------------------------------------------------------------
   Add this value to the C array 'arrayP' with key 'key'.
 ----------------------------------------------------------------------------*/
+    this->validateInstantiated();
+
     env_wrap env;
 
     xmlrpc_struct_set_value_n(&env.env_c, structP,
@@ -186,8 +237,11 @@ value::addToCStruct(xmlrpc_value * const structP,
 
 value::type_t 
 value::type() const {
+
+    this->validateInstantiated();
+
     /* You'd think we could just cast from xmlrpc_type to
-       value:type_t, but Gcc warns if we do that.  So we have to do this
+       value::type_t, but Gcc warns if we do that.  So we have to do this
        even messier union nonsense.
     */
     union {
@@ -202,6 +256,15 @@ value::type() const {
 
 
 
+ostream& operator<<(ostream& out, value::type_t const& type) {
+
+    string typeName;
+
+    return out << string(xmlrpc_type_name((xmlrpc_type)type));
+}
+
+
+
 value_int::value_int(int const cppvalue) {
 
     class cWrapper {
@@ -239,6 +302,8 @@ value_int::value_int(xmlrpc_c::value const baseValue) {
 
 value_int::operator int() const {
 
+    this->validateInstantiated();
+
     int retval;
     env_wrap env;
 
@@ -250,6 +315,14 @@ value_int::operator int() const {
 
 
 
+int
+value_int::cvalue() const {
+
+    return static_cast<int>(*this);
+}
+
+
+
 value_double::value_double(double const cppvalue) {
 
     class cWrapper {
@@ -285,6 +358,8 @@ value_double::value_double(xmlrpc_c::value const baseValue) {
 
 value_double::operator double() const {
 
+    this->validateInstantiated();
+
     double retval;
 
     env_wrap env;
@@ -297,6 +372,14 @@ value_double::operator double() const {
 
 
 
+double
+value_double::cvalue() const {
+
+    return static_cast<double>(*this);
+}
+
+
+
 value_boolean::value_boolean(bool const cppvalue) {
 
     class cWrapper {
@@ -321,8 +404,21 @@ value_boolean::value_boolean(bool const cppvalue) {
 
 
 
+value_boolean::value_boolean(xmlrpc_c::value const baseValue) {
+
+    if (baseValue.type() != xmlrpc_c::value::TYPE_BOOLEAN)
+        throw(error("Not boolean type.  See type() method"));
+    else {
+        this->instantiate(baseValue.cValueP);
+    }
+}
+
+
+
 value_boolean::operator bool() const {
 
+    this->validateInstantiated();
+
     xmlrpc_bool retval;
 
     env_wrap env;
@@ -330,18 +426,15 @@ value_boolean::operator bool() const {
     xmlrpc_read_bool(&env.env_c, this->cValueP, &retval);
     throwIfError(env);
 
-    return (bool)retval;
+    return (retval != false);
 }
 
 
 
-value_boolean::value_boolean(xmlrpc_c::value const baseValue) {
+bool
+value_boolean::cvalue() const {
 
-    if (baseValue.type() != xmlrpc_c::value::TYPE_BOOLEAN)
-        throw(error("Not boolean type.  See type() method"));
-    else {
-        this->instantiate(baseValue.cValueP);
-    }
+    return static_cast<bool>(*this);
 }
 
 
@@ -371,6 +464,15 @@ value_datetime::value_datetime(string const cppvalue) {
 
 
 
+value_datetime::value_datetime(xmlrpc_datetime const cppvalue) {
+
+    cDatetimeValueWrapper wrapper(cppvalue);
+    
+    this->instantiate(wrapper.valueP);
+}
+
+
+
 value_datetime::value_datetime(time_t const cppvalue) {
 
     cDatetimeValueWrapper wrapper(cppvalue);
@@ -380,21 +482,25 @@ value_datetime::value_datetime(time_t const cppvalue) {
 
 
 
+#if XMLRPC_HAVE_TIMEVAL
 value_datetime::value_datetime(struct timeval const& cppvalue) {
 
-    cDatetimeValueWrapper wrapper(cppvalue.tv_sec);
+    cDatetimeValueWrapper wrapper(cppvalue);
 
     this->instantiate(wrapper.valueP);
 }
+#endif
 
 
 
+#if XMLRPC_HAVE_TIMESPEC
 value_datetime::value_datetime(struct timespec const& cppvalue) {
 
-    cDatetimeValueWrapper wrapper(cppvalue.tv_sec);
+    cDatetimeValueWrapper wrapper(cppvalue);
 
     this->instantiate(wrapper.valueP);
 }
+#endif
 
 
 
@@ -409,8 +515,25 @@ value_datetime::value_datetime(xmlrpc_c::value const baseValue) {
 
 
 
+value_datetime::operator xmlrpc_datetime() const {
+
+    this->validateInstantiated();
+
+    xmlrpc_datetime retval;
+    env_wrap env;
+
+    xmlrpc_read_datetime(&env.env_c, this->cValueP, &retval);
+    throwIfError(env);
+
+    return retval;
+}
+
+
+
 value_datetime::operator time_t() const {
 
+    this->validateInstantiated();
+
     time_t retval;
     env_wrap env;
 
@@ -422,30 +545,124 @@ value_datetime::operator time_t() const {
 
 
 
-value_string::value_string(string const& cppvalue) {
-    
-    class cWrapper {
-    public:
-        xmlrpc_value * valueP;
+#if XMLRPC_HAVE_TIMEVAL
+
+value_datetime::operator timeval() const {
+
+    this->validateInstantiated();
+
+    struct timeval retval;
+    env_wrap env;
+
+    xmlrpc_read_datetime_timeval(&env.env_c, this->cValueP, &retval);
+    throwIfError(env);
+
+    return retval;
+}
+#endif
+
+
+
+#if XMLRPC_HAVE_TIMESPEC
+
+value_datetime::operator timespec() const {
+
+    this->validateInstantiated();
+
+    struct timespec retval;
+    env_wrap env;
+
+    xmlrpc_read_datetime_timespec(&env.env_c, this->cValueP, &retval);
+    throwIfError(env);
+
+    return retval;
+}
+#endif
+
+
+
+time_t
+value_datetime::cvalue() const {
+
+    return static_cast<time_t>(*this);
+}
+
+
+
+string
+value_datetime::iso8601Value() const {
+
+    string retval;
+
+    this->validateInstantiated();
+
+    const char * iso8601;
+    env_wrap env;
+
+    xmlrpc_read_datetime_8601(&env.env_c, this->cValueP, &iso8601);
+    throwIfError(env);
+
+    retval = iso8601;
+
+    xmlrpc_strfree(iso8601);
+
+    return retval;
+}
+
+
+
+class cNewStringWrapper {
+public:
+    xmlrpc_value * valueP;
         
-        cWrapper(string const cppvalue) {
-            env_wrap env;
+    cNewStringWrapper(string               const cppvalue,
+                      value_string::nlCode const nlCode) {
+        env_wrap env;
             
-            this->valueP = xmlrpc_string_new(&env.env_c, cppvalue.c_str());
-            throwIfError(env);
+        switch (nlCode) {
+        case value_string::nlCode_all:
+            this->valueP = xmlrpc_string_new_lp(&env.env_c,
+                                                cppvalue.length(),
+                                                cppvalue.c_str());
+            break;
+        case value_string::nlCode_lf:
+            this->valueP = xmlrpc_string_new_lp_cr(&env.env_c,
+                                                   cppvalue.length(),
+                                                   cppvalue.c_str());
+            break;
+        default:
+            throw(error("Newline encoding argument to value_string "
+                        "constructor is not one of the defined "
+                        "enumerations of value_string::nlCode"));
         }
-        ~cWrapper() {
-            xmlrpc_DECREF(this->valueP);
-        }
-    };
+        throwIfError(env);
+    }
+    ~cNewStringWrapper() {
+        xmlrpc_DECREF(this->valueP);
+    }
+};
     
-    cWrapper wrapper(cppvalue);
+
+
+value_string::value_string(std::string          const& cppvalue,
+                           value_string::nlCode const  nlCode) {
+    
+    cNewStringWrapper wrapper(cppvalue, nlCode);
     
     this->instantiate(wrapper.valueP);
 }
 
 
 
+value_string::value_string(std::string const& cppvalue) {
+
+    cNewStringWrapper wrapper(cppvalue, nlCode_all);
+    
+    this->instantiate(wrapper.valueP);
+}
+
+
+    
 value_string::value_string(xmlrpc_c::value const baseValue) {
 
     if (baseValue.type() != xmlrpc_c::value::TYPE_STRING)
@@ -457,9 +674,36 @@ value_string::value_string(xmlrpc_c::value const baseValue) {
 
 
 
+std::string
+value_string::crlfValue() const {
+
+    class cWrapper {
+    public:
+        const char * str;
+        size_t length;
+        cWrapper(xmlrpc_value * valueP) {
+            env_wrap env;
+            
+            xmlrpc_read_string_lp_crlf(&env.env_c, valueP, &length, &str);
+            throwIfError(env);
+        }
+        ~cWrapper() {
+            free((char*)str);
+        }
+    };
+    
+    this->validateInstantiated();
+
+    cWrapper wrapper(this->cValueP);
+
+    return string(wrapper.str, wrapper.length);
+}
+
+
+
 value_string::operator string() const {
 
-    env_wrap env;
+    this->validateInstantiated();
 
     cStringWrapper adapter(this->cValueP);
 
@@ -468,6 +712,14 @@ value_string::operator string() const {
 
 
 
+std::string
+value_string::cvalue() const {
+
+    return static_cast<std::string>(*this);
+}
+
+
+
 value_bytestring::value_bytestring(
     vector<unsigned char> const& cppvalue) {
 
@@ -494,6 +746,17 @@ value_bytestring::value_bytestring(
 
 
 
+value_bytestring::value_bytestring(xmlrpc_c::value const baseValue) {
+
+    if (baseValue.type() != xmlrpc_c::value::TYPE_BYTESTRING)
+        throw(error("Not byte string type.  See type() method"));
+    else {
+        this->instantiate(baseValue.cValueP);
+    }
+}
+
+
+
 vector<unsigned char>
 value_bytestring::vectorUcharValue() const {
 
@@ -513,6 +776,8 @@ value_bytestring::vectorUcharValue() const {
         }
     };
     
+    this->validateInstantiated();
+
     cWrapper wrapper(this->cValueP);
 
     return vector<unsigned char>(&wrapper.contents[0], 
@@ -521,9 +786,19 @@ value_bytestring::vectorUcharValue() const {
 
 
 
+vector<unsigned char>
+value_bytestring::cvalue() const {
+
+    return this->vectorUcharValue();
+}
+
+
+
 size_t
 value_bytestring::length() const {
 
+    this->validateInstantiated();
+
     env_wrap env;
     size_t length;
 
@@ -535,17 +810,6 @@ value_bytestring::length() const {
 
 
 
-value_bytestring::value_bytestring(xmlrpc_c::value const baseValue) {
-
-    if (baseValue.type() != xmlrpc_c::value::TYPE_BYTESTRING)
-        throw(error("Not byte string type.  See type() method"));
-    else {
-        this->instantiate(baseValue.cValueP);
-    }
-}
-
-
-
 value_array::value_array(vector<xmlrpc_c::value> const& cppvalue) {
     
     class cWrapper {
@@ -588,6 +852,8 @@ value_array::value_array(xmlrpc_c::value const baseValue) {
 vector<xmlrpc_c::value>
 value_array::vectorValueValue() const {
 
+    this->validateInstantiated();
+
     env_wrap env;
 
     unsigned int arraySize;
@@ -626,9 +892,19 @@ value_array::vectorValueValue() const {
 
 
 
+vector<xmlrpc_c::value>
+value_array::cvalue() const {
+
+    return this->vectorValueValue();
+}
+
+
+
 size_t
 value_array::size() const {
 
+    this->validateInstantiated();
+
     env_wrap env;
     unsigned int arraySize;
 
@@ -685,6 +961,8 @@ value_struct::value_struct(xmlrpc_c::value const baseValue) {
 
 value_struct::operator map<string, xmlrpc_c::value>() const {
 
+    this->validateInstantiated();
+
     env_wrap env;
     unsigned int structSize;
 
@@ -729,6 +1007,14 @@ value_struct::operator map<string, xmlrpc_c::value>() const {
 
 
 
+map<string, xmlrpc_c::value>
+value_struct::cvalue() const {
+
+    return static_cast<map<string, xmlrpc_c::value> >(*this);
+}
+
+
+
 value_nil::value_nil() {
     
     class cWrapper {
@@ -764,5 +1050,70 @@ value_nil::value_nil(xmlrpc_c::value const baseValue) {
 
 
 
-} // namespace
+void *
+value_nil::cvalue() const {
 
+    return NULL;
+}
+
+
+
+value_i8::value_i8(xmlrpc_int64 const cppvalue) {
+
+    class cWrapper {
+    public:
+        xmlrpc_value * valueP;
+        
+        cWrapper(xmlrpc_int64 const cppvalue) {
+            env_wrap env;
+            
+            this->valueP = xmlrpc_i8_new(&env.env_c, cppvalue);
+            throwIfError(env);
+        }
+        ~cWrapper() {
+            xmlrpc_DECREF(this->valueP);
+        }
+    };
+    
+    cWrapper wrapper(cppvalue);
+    
+    this->instantiate(wrapper.valueP);
+}
+
+
+
+value_i8::value_i8(xmlrpc_c::value const baseValue) {
+
+    if (baseValue.type() != xmlrpc_c::value::TYPE_I8)
+        throw(error("Not 64 bit integer type.  See type() method"));
+    else {
+        this->instantiate(baseValue.cValueP);
+    }
+}
+
+
+
+value_i8::operator xmlrpc_int64() const {
+
+    this->validateInstantiated();
+
+    xmlrpc_int64 retval;
+    env_wrap env;
+
+    xmlrpc_read_i8(&env.env_c, this->cValueP, &retval);
+    throwIfError(env);
+
+    return retval;
+}
+
+
+
+xmlrpc_int64
+value_i8::cvalue() const {
+
+    return static_cast<xmlrpc_int64>(*this);
+}
+
+
+
+} // namespace
