@@ -100,11 +100,11 @@ accessStringValue(xmlrpc_env *         const envP,
     
     validateStringType(envP, valueP);
     if (!envP->fault_occurred) {
-        unsigned int const size = 
+        size_t const size = 
             XMLRPC_MEMBLOCK_SIZE(char, &valueP->_block);
         const char * const contents = 
             XMLRPC_MEMBLOCK_CONTENTS(char, &valueP->_block);
-        unsigned int const len = size - 1;
+        size_t const len = size - 1;
             /* The memblock has a null character added to the end */
 
         verifyNoNulls(envP, contents, len);
@@ -141,7 +141,7 @@ xmlrpc_read_string(xmlrpc_env *         const envP,
         MALLOCARRAY(stringValue, length + 1);
         if (stringValue == NULL)
             xmlrpc_faultf(envP, "Unable to allocate space "
-                          "for %u-character string", length);
+                          "for %u-character string", (unsigned)length);
         else {
             memcpy(stringValue, contents, length);
             stringValue[length] = '\0';
@@ -189,7 +189,7 @@ copyAndConvertLfToCrlf(xmlrpc_env *  const envP,
     MALLOCARRAY(dst, dstLen + 1);
     if (dst == NULL)
         xmlrpc_faultf(envP, "Unable to allocate space "
-                      "for %u-character string", dstLen + 1);
+                      "for %u-character string", (unsigned)dstLen + 1);
     else {
         const char * p;  /* source pointer */
         char * q;        /* destination pointer */
@@ -238,7 +238,15 @@ void
 xmlrpc_read_string_old(xmlrpc_env *         const envP,
                        const xmlrpc_value * const valueP,
                        const char **        const stringValueP) {
+/*----------------------------------------------------------------------------
+   Like xmlrpc_read_string(), except it returns as *stringValueP a pointer
+   into memory owned by *valueP, rather than new memory to be owned by
+   Caller.
 
+   This is for internal use; it's necessary to implement the deprecated
+   xmlrpc_parse_value(), which also returns pointers to someone else's
+   storage.
+-----------------------------------------------------------------------------*/
     size_t length;
     accessStringValue(envP, valueP, &length, stringValueP);
 }
@@ -253,7 +261,7 @@ xmlrpc_read_string_lp(xmlrpc_env *         const envP,
 
     validateStringType(envP, valueP);
     if (!envP->fault_occurred) {
-        unsigned int const size = 
+        size_t const size = 
             XMLRPC_MEMBLOCK_SIZE(char, &valueP->_block);
         const char * const contents = 
             XMLRPC_MEMBLOCK_CONTENTS(char, &valueP->_block);
@@ -263,7 +271,7 @@ xmlrpc_read_string_lp(xmlrpc_env *         const envP,
         stringValue = malloc(size);
         if (stringValue == NULL)
             xmlrpc_faultf(envP, "Unable to allocate %u bytes for string.",
-                          size);
+                          (unsigned int)size);
         else {
             memcpy(stringValue, contents, size);
             *stringValueP = stringValue;
@@ -282,7 +290,7 @@ xmlrpc_read_string_lp_crlf(xmlrpc_env *         const envP,
 
     validateStringType(envP, valueP);
     if (!envP->fault_occurred) {
-        unsigned int const size = 
+        size_t const size = 
             XMLRPC_MEMBLOCK_SIZE(char, &valueP->_block); /* Includes NUL */
         const char * const contents = 
             XMLRPC_MEMBLOCK_CONTENTS(char, &valueP->_block);
@@ -299,7 +307,10 @@ xmlrpc_read_string_lp_old(xmlrpc_env *         const envP,
                           const xmlrpc_value * const valueP,
                           size_t *             const lengthP,
                           const char **        const stringValueP) {
-
+/*----------------------------------------------------------------------------
+  This is to xmlrpc_read_string_lp() as xmlrpc_read_string_old() is
+  to xmlrpc_read_string().
+-----------------------------------------------------------------------------*/
     validateStringType(envP, valueP);
     if (!envP->fault_occurred) {
         *lengthP =      XMLRPC_MEMBLOCK_SIZE(char, &valueP->_block) - 1;
@@ -371,7 +382,7 @@ xmlrpc_read_string_w(xmlrpc_env *     const envP,
         MALLOCARRAY(stringValue, length + 1);
         if (stringValue == NULL)
             xmlrpc_faultf(envP, "Unable to allocate space for %u-byte string", 
-                          length);
+                          (unsigned)length);
         else {
             memcpy(stringValue, wcontents, length * sizeof(wchar_t));
             stringValue[length] = '\0';
@@ -422,7 +433,7 @@ wCopyAndConvertLfToCrlf(xmlrpc_env *     const envP,
     MALLOCARRAY(dst, dstLen + 1);
     if (dst == NULL)
         xmlrpc_faultf(envP, "Unable to allocate space "
-                      "for %u-character string", dstLen + 1);
+                      "for %u-character string", (unsigned)dstLen + 1);
     else {
         const wchar_t * p;  /* source pointer */
         wchar_t * q;        /* destination pointer */
@@ -468,7 +479,10 @@ void
 xmlrpc_read_string_w_old(xmlrpc_env *     const envP,
                          xmlrpc_value *   const valueP,
                          const wchar_t ** const stringValueP) {
-
+/*----------------------------------------------------------------------------
+  This is to xmlrpc_read_string_w() as xmlrpc_read_string_old() is
+  to xmlrpc_read_string().
+-----------------------------------------------------------------------------*/
     size_t length;
 
     accessStringValueW(envP, valueP, &length, stringValueP);
@@ -540,7 +554,10 @@ xmlrpc_read_string_w_lp_old(xmlrpc_env *     const envP,
                             xmlrpc_value *   const valueP,
                             size_t *         const lengthP,
                             const wchar_t ** const stringValueP) {
-
+/*----------------------------------------------------------------------------
+  This is to xmlrpc_read_string_w_lp() as xmlrpc_read_string_old() is
+  to xmlrpc_read_string().
+-----------------------------------------------------------------------------*/
     validateStringType(envP, valueP);
     if (!envP->fault_occurred) {
         setupWcsBlock(envP, valueP);
@@ -560,18 +577,6 @@ xmlrpc_read_string_w_lp_old(xmlrpc_env *     const envP,
 
 
 
-static void
-validateUtf(xmlrpc_env * const envP,
-            const char * const value,
-            size_t       const length) {
-
-#if HAVE_UNICODE_WCHAR
-    xmlrpc_validate_utf8(envP, value, length);
-#endif
-}
-
-
-
 static void
 copyLines(xmlrpc_env *       const envP,
           const char *       const src,
@@ -677,15 +682,16 @@ copySimple(xmlrpc_env *       const envP,
 
 enum crTreatment { CR_IS_LINEDELIM, CR_IS_CHAR };
 
-static xmlrpc_value *
+static void
 stringNew(xmlrpc_env *     const envP, 
           size_t           const length,
           const char *     const value,
-          enum crTreatment const crTreatment) {
+          enum crTreatment const crTreatment,
+          xmlrpc_value **  const valPP) {
 
     xmlrpc_value * valP;
 
-    validateUtf(envP, value, length);
+    xmlrpc_validate_utf8(envP, value, length);
 
     if (!envP->fault_occurred) {
         xmlrpc_createXmlrpcValue(envP, &valP);
@@ -704,9 +710,10 @@ stringNew(xmlrpc_env *     const envP,
 
             if (envP->fault_occurred)
                 free(valP);
+            else
+                *valPP = valP;
         }
     }
-    return valP;
 }
 
 
@@ -716,7 +723,9 @@ xmlrpc_string_new_lp(xmlrpc_env * const envP,
                      size_t       const length,
                      const char * const value) {
 
-    return stringNew(envP, length, value, CR_IS_LINEDELIM);
+    xmlrpc_value * retval;
+    stringNew(envP, length, value, CR_IS_LINEDELIM, &retval);
+    return retval;
 }
 
 
@@ -726,7 +735,9 @@ xmlrpc_string_new_lp_cr(xmlrpc_env * const envP,
                         size_t       const length,
                         const char * const value) {
 
-    return stringNew(envP, length, value, CR_IS_CHAR);
+    xmlrpc_value * retval;
+    stringNew(envP, length, value, CR_IS_CHAR, &retval);
+    return retval;
 }
 
 
@@ -735,7 +746,9 @@ xmlrpc_value *
 xmlrpc_string_new(xmlrpc_env * const envP,
                   const char * const value) {
     
-    return stringNew(envP, strlen(value), value, CR_IS_LINEDELIM);
+    xmlrpc_value * retval;
+    stringNew(envP, strlen(value), value, CR_IS_LINEDELIM, &retval);
+    return retval;
 }
 
 
@@ -744,7 +757,9 @@ xmlrpc_value *
 xmlrpc_string_new_cr(xmlrpc_env * const envP,
                      const char * const value) {
 
-    return stringNew(envP, strlen(value), value, CR_IS_CHAR);
+    xmlrpc_value * retval;
+    stringNew(envP, strlen(value), value, CR_IS_CHAR, &retval);
+    return retval;
 }
 
 
@@ -762,7 +777,7 @@ xmlrpc_string_new_va(xmlrpc_env * const envP,
     
     xmlrpc_vasprintf(&formattedString, format, args);
 
-    if (formattedString == xmlrpc_strsol) {
+    if (xmlrpc_strnomem(formattedString)) {
         xmlrpc_faultf(envP, "Out of memory building formatted string");
         retvalP = NULL;  /* defeat compiler warning */
     } else
@@ -796,29 +811,26 @@ xmlrpc_string_new_f(xmlrpc_env * const envP,
 
 #if HAVE_UNICODE_WCHAR
 
-static xmlrpc_value *
+static void
 stringWNew(xmlrpc_env *     const envP, 
            size_t           const length,
            const wchar_t *  const value,
-           enum crTreatment const crTreatment) {
+           enum crTreatment const crTreatment,
+           xmlrpc_value **  const valPP) {
 
-    xmlrpc_value * valP;
     xmlrpc_mem_block * utf8P;
 
-    valP = NULL;  /* defeat compiler warning */
-
     utf8P = xmlrpc_wcs_to_utf8(envP, value, length);
     if (!envP->fault_occurred) {
         char * const utf8_value = XMLRPC_MEMBLOCK_CONTENTS(char, utf8P);
         size_t const utf8_len   = XMLRPC_MEMBLOCK_SIZE(char, utf8P);
         
         if (!envP->fault_occurred) {
-            valP = stringNew(envP, utf8_len, utf8_value, crTreatment);
+            stringNew(envP, utf8_len, utf8_value, crTreatment, valPP);
 
             XMLRPC_MEMBLOCK_FREE(char, utf8P);
         }
     }
-    return valP;
 }
 
 
@@ -828,7 +840,9 @@ xmlrpc_string_w_new_lp(xmlrpc_env *    const envP,
                        size_t          const length,
                        const wchar_t * const value) {
 
-    return stringWNew(envP, length, value, CR_IS_LINEDELIM);
+    xmlrpc_value * valP;
+    stringWNew(envP, length, value, CR_IS_LINEDELIM, &valP);
+    return valP;
 }
 
 
@@ -839,7 +853,9 @@ xmlrpc_string_w_new_lp_cr(xmlrpc_env *    const envP,
                           size_t          const length,
                           const wchar_t * const value) {
 
-    return stringWNew(envP, length, value, CR_IS_CHAR);
+    xmlrpc_value * valP;
+    stringWNew(envP, length, value, CR_IS_CHAR, &valP);
+    return valP;
 }
 
 
@@ -849,7 +865,9 @@ xmlrpc_value *
 xmlrpc_string_w_new(xmlrpc_env *    const envP,
                     const wchar_t * const value) {
 
-    return stringWNew(envP, wcslen(value), value, CR_IS_LINEDELIM);
+    xmlrpc_value * valP;
+    stringWNew(envP, wcslen(value), value, CR_IS_LINEDELIM, &valP);
+    return valP;
 }
 
 
@@ -858,7 +876,9 @@ xmlrpc_value *
 xmlrpc_string_w_new_cr(xmlrpc_env *    const envP,
                        const wchar_t * const value) {
 
-    return stringWNew(envP, wcslen(value), value, CR_IS_CHAR);
+    xmlrpc_value * valP;
+    stringWNew(envP, wcslen(value), value, CR_IS_CHAR, &valP);
+    return valP;
 }
 
 #endif   /* HAVE_UNICODE_WCHAR */
