@@ -1,30 +1,6 @@
-/* Copyright (C) 2001 by First Peer, Inc. All rights reserved.
-**
-** Redistribution and use in source and binary forms, with or without
-** modification, are permitted provided that the following conditions
-** are met:
-** 1. Redistributions of source code must retain the above copyright
-**    notice, this list of conditions and the following disclaimer.
-** 2. Redistributions in binary form must reproduce the above copyright
-**    notice, this list of conditions and the following disclaimer in the
-**    documentation and/or other materials provided with the distribution.
-** 3. The name of the author may not be used to endorse or promote products
-**    derived from this software without specific prior written permission. 
-**  
-** THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
-** ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-** ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
-** FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
-** DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
-** OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
-** HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
-** LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
-** OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
-** SUCH DAMAGE. */
-
 #include "xmlrpc_config.h"
 
+#include <assert.h>
 #include <stddef.h>
 #include <stdlib.h>
 #include <string.h>
@@ -95,22 +71,28 @@ xmlrpc_struct_new(xmlrpc_env * const envP) {
 */
 
 int 
-xmlrpc_struct_size(xmlrpc_env* env, xmlrpc_value* strct)
-{
+xmlrpc_struct_size(xmlrpc_env *   const envP,
+                   xmlrpc_value * const structP) {
+
     int retval;
 
-    /* Suppress a compiler warning about uninitialized variables. */
-    retval = 0;
+    XMLRPC_ASSERT_ENV_OK(envP);
+    XMLRPC_ASSERT_VALUE_OK(structP);
 
-    XMLRPC_ASSERT_ENV_OK(env);
-    XMLRPC_ASSERT_VALUE_OK(strct);
+    if (structP->_type != XMLRPC_TYPE_STRUCT) {
+        xmlrpc_env_set_fault_formatted(
+            envP, XMLRPC_TYPE_ERROR, "Value is not a struct.  It is type #%d",
+            structP->_type);
+        retval = -1;
+    } else {
+        size_t const size =
+            XMLRPC_MEMBLOCK_SIZE(_struct_member, &structP->_block);
 
-    XMLRPC_TYPE_CHECK(env, strct, XMLRPC_TYPE_STRUCT);
-    retval = XMLRPC_MEMBLOCK_SIZE(_struct_member, &strct->_block);
+        assert((size_t)(int)size == size);
+            /* Because structs are defined to have few enough members */
 
- cleanup:
-    if (env->fault_occurred)
-        return -1;
+        retval = (int)size;
+    }
     return retval;
 }
 
@@ -137,42 +119,48 @@ hashStructKey(const char * const key,
 
 
 
-/*=========================================================================
-**  find_member
-**=========================================================================
-**  Get the index of the member with the specified key, or -1 if no such
-**  member exists.
-*/
-
-static int 
-find_member(xmlrpc_value * const strctP, 
-            const char *   const key, 
-            size_t         const keyLen) {
+static void
+findMember(xmlrpc_value * const structP, 
+           const char *   const key, 
+           size_t         const keyLen,
+           bool *         const foundP,
+           unsigned int * const indexP) {
 
     size_t size, i;
     uint32_t searchHash;
     _struct_member * contents;  /* array */
-    xmlrpc_value * keyvalP;
-    const char * keystr;
-    size_t keystrSize;
+    bool found;
+    size_t foundIndex;  /* Meaningful only when 'found' is true */
 
-    XMLRPC_ASSERT_VALUE_OK(strctP);
+    XMLRPC_ASSERT_VALUE_OK(structP);
     XMLRPC_ASSERT(key != NULL);
+    foundIndex = 0;  /* defeat used-before-set compiler warning */
 
     /* Look for our key. */
     searchHash = hashStructKey(key, keyLen);
-    size = XMLRPC_MEMBLOCK_SIZE(_struct_member, &strctP->_block);
-    contents = XMLRPC_MEMBLOCK_CONTENTS(_struct_member, &strctP->_block);
-    for (i = 0; i < size; ++i) {
+    size = XMLRPC_MEMBLOCK_SIZE(_struct_member, &structP->_block);
+    contents = XMLRPC_MEMBLOCK_CONTENTS(_struct_member, &structP->_block);
+    for (i = 0, found = false; i < size && !found; ++i) {
         if (contents[i].keyHash == searchHash) {
-            keyvalP = contents[i].key;
-            keystr = XMLRPC_MEMBLOCK_CONTENTS(char, &keyvalP->_block);
-            keystrSize = XMLRPC_MEMBLOCK_SIZE(char, &keyvalP->_block)-1;
-            if (keystrSize == keyLen && memcmp(key, keystr, keyLen) == 0)
-                return i;
+            xmlrpc_value * const keyvalP = contents[i].key;
+            const char * const keystr =
+                XMLRPC_MEMBLOCK_CONTENTS(char, &keyvalP->_block);
+            size_t const keystrSize =
+                XMLRPC_MEMBLOCK_SIZE(char, &keyvalP->_block)-1;
+            if (keystrSize == keyLen && memcmp(key, keystr, keyLen) == 0) {
+                found = true;
+                foundIndex = i;
+            }
         }   
     }
-    return -1;
+    if (found) {
+        assert((size_t)(int)foundIndex == foundIndex);
+            /* Definition of structure says it has few enough members */
+
+        if (indexP)
+            *indexP = foundIndex;
+    }
+    *foundP = found;
 }
 
 
@@ -195,25 +183,29 @@ xmlrpc_struct_has_key(xmlrpc_env *   const envP,
 
 int 
 xmlrpc_struct_has_key_n(xmlrpc_env   * const envP,
-                        xmlrpc_value * const strctP,
+                        xmlrpc_value * const structP,
                         const char *   const key, 
-                        size_t         const key_len) {
-    int index;
+                        size_t         const keyLen) {
+    int retval;
 
     /* Suppress a compiler warning about uninitialized variables. */
-    index = 0;
+    retval = 0;
 
     XMLRPC_ASSERT_ENV_OK(envP);
-    XMLRPC_ASSERT_VALUE_OK(strctP);
+    XMLRPC_ASSERT_VALUE_OK(structP);
     XMLRPC_ASSERT(key != NULL);
     
-    XMLRPC_TYPE_CHECK(envP, strctP, XMLRPC_TYPE_STRUCT);
-    index = find_member(strctP, key, key_len);
+    if (structP->_type != XMLRPC_TYPE_STRUCT)
+        xmlrpc_env_set_fault(envP, XMLRPC_TYPE_ERROR,
+                             "Value is not a struct");
+    else {
+        bool found;
 
- cleanup:
-    if (envP->fault_occurred)
-        return 0;
-    return (index >= 0);
+        findMember(structP, key, keyLen, &found, NULL);
+
+        retval = found ? 1 : 0;
+    }
+    return retval;
 }
 
 
@@ -248,11 +240,12 @@ xmlrpc_struct_find_value(xmlrpc_env *    const envP,
             envP, XMLRPC_TYPE_ERROR, "Value is not a struct.  It is type #%d",
             structP->_type);
     else {
-        int index;
+        bool found;
+        unsigned int index;
 
         /* Get our member index. */
-        index = find_member(structP, key, strlen(key));
-        if (index < 0)
+        findMember(structP, key, strlen(key), &found, &index);
+        if (!found)
             *valuePP = NULL;
         else {
             _struct_member * const members =
@@ -292,13 +285,15 @@ xmlrpc_struct_find_value_v(xmlrpc_env *    const envP,
                 "It is type #%d",
                 keyP->_type);
         else {
-            int index;
+            bool found;
+            unsigned int index;
 
             /* Get our member index. */
-            index = find_member(structP, 
-                                XMLRPC_MEMBLOCK_CONTENTS(char, &keyP->_block),
-                                XMLRPC_MEMBLOCK_SIZE(char, &keyP->_block)-1);
-            if (index < 0)
+            findMember(structP, 
+                       XMLRPC_MEMBLOCK_CONTENTS(char, &keyP->_block),
+                       XMLRPC_MEMBLOCK_SIZE(char, &keyP->_block)-1,
+                       &found, &index);
+            if (!found)
                 *valuePP = NULL;
             else {
                 _struct_member * const members =
@@ -377,9 +372,9 @@ xmlrpc_struct_get_value_n(xmlrpc_env *   const envP,
                           size_t         const keyLen) {
 
     xmlrpc_value * retval;
-    xmlrpc_value * keyP;
+    xmlrpc_value * keyP;  /* 'key' as an XML-RPC string */
     
-    keyP = xmlrpc_build_value(envP, "s#", key, keyLen);
+    keyP = xmlrpc_string_new_lp(envP, keyLen, key);
     if (!envP->fault_occurred) {
         xmlrpc_struct_find_value_v(envP, structP, keyP, &retval);
 
@@ -447,10 +442,9 @@ xmlrpc_struct_set_value_n(xmlrpc_env *    const envP,
             "Type is %d; struct is %d",
             xmlrpc_value_type(strctP), XMLRPC_TYPE_STRUCT);
     else {
-        xmlrpc_value * keyvalP;
+        xmlrpc_value * keyvalP;  /* 'key' as an XML-RPC string */
 
-        /* Get the key as an xmlrpc_value */
-        keyvalP = xmlrpc_build_value(envP, "s#", key, keyLen);
+        keyvalP = xmlrpc_string_new_lp(envP, keyLen, key);
         if (!envP->fault_occurred)
             xmlrpc_struct_set_value_v(envP, strctP, keyvalP, valueP);
 
@@ -460,56 +454,91 @@ xmlrpc_struct_set_value_n(xmlrpc_env *    const envP,
 
 
 
-void 
-xmlrpc_struct_set_value_v(xmlrpc_env *   const envP,
-                          xmlrpc_value * const strctP,
-                          xmlrpc_value * const keyvalP,
-                          xmlrpc_value * const valueP) {
+static void
+changeMemberValue(xmlrpc_value * const structP,
+                  unsigned int   const mbrIndex,
+                  xmlrpc_value * const newValueP) {
+/*----------------------------------------------------------------------------
+  Change the value of an existing member. (But be careful--the original and
+  new values might be the same object, so watch the order of INCREF and DECREF
+  calls!)
+-----------------------------------------------------------------------------*/
+    _struct_member * const members =
+        XMLRPC_MEMBLOCK_CONTENTS(_struct_member, &structP->_block);
+    _struct_member * const memberP = &members[mbrIndex];
+    xmlrpc_value * const oldValueP = memberP->value;
 
-    char *key;
-    size_t key_len;
-    int index;
-    _struct_member *members, *member, new_member;
-    xmlrpc_value *old_value;
+    /* Juggle our references. */
+    memberP->value = newValueP;
+    xmlrpc_INCREF(memberP->value);
+    xmlrpc_DECREF(oldValueP);
+}
 
-    XMLRPC_ASSERT_ENV_OK(envP);
-    XMLRPC_ASSERT_VALUE_OK(strctP);
-    XMLRPC_ASSERT_VALUE_OK(keyvalP);
-    XMLRPC_ASSERT_VALUE_OK(valueP);
 
-    XMLRPC_TYPE_CHECK(envP, strctP, XMLRPC_TYPE_STRUCT);
-    XMLRPC_TYPE_CHECK(envP, keyvalP, XMLRPC_TYPE_STRING);
 
-    key = XMLRPC_MEMBLOCK_CONTENTS(char, &keyvalP->_block);
-    key_len = XMLRPC_MEMBLOCK_SIZE(char, &keyvalP->_block) - 1;
-    index = find_member(strctP, key, key_len);
+static void
+addNewMember(xmlrpc_env *   const envP,
+             xmlrpc_value * const structP,
+             xmlrpc_value * const keyvalP,
+             xmlrpc_value * const valueP) {
+/*----------------------------------------------------------------------------
+   Add a new member.  Assume no member already exists with this key.
+-----------------------------------------------------------------------------*/
+    const char * const key =
+        XMLRPC_MEMBLOCK_CONTENTS(char, &keyvalP->_block);
+    size_t const keyLen =
+        XMLRPC_MEMBLOCK_SIZE(char, &keyvalP->_block) - 1;
 
-    if (index >= 0) {
-        /* Change the value of an existing member. (But be careful--the
-        ** original and new values might be the same object, so watch
-        ** the order of INCREF and DECREF calls!) */
-        members = XMLRPC_MEMBLOCK_CONTENTS(_struct_member, &strctP->_block);
-        member = &members[index];
+    _struct_member newMember;
 
-        /* Juggle our references. */
-        old_value = member->value;
-        member->value = valueP;
-        xmlrpc_INCREF(member->value);
-        xmlrpc_DECREF(old_value);
-    } else {
-        /* Add a new member. */
-        new_member.keyHash = hashStructKey(key, key_len);
-        new_member.key     = keyvalP;
-        new_member.value   = valueP;
-        XMLRPC_MEMBLOCK_APPEND(_struct_member, envP, &strctP->_block,
-                               &new_member, 1);
-        XMLRPC_FAIL_IF_FAULT(envP);
+    newMember.keyHash = hashStructKey(key, keyLen);
+    newMember.key     = keyvalP;
+    newMember.value   = valueP;
+
+    XMLRPC_MEMBLOCK_APPEND(_struct_member, envP, &structP->_block,
+                           &newMember, 1);
+
+    if (!envP->fault_occurred) {
         xmlrpc_INCREF(keyvalP);
         xmlrpc_INCREF(valueP);
     }
+}
 
-cleanup:
-    return;
+
+
+void 
+xmlrpc_struct_set_value_v(xmlrpc_env *   const envP,
+                          xmlrpc_value * const structP,
+                          xmlrpc_value * const keyvalP,
+                          xmlrpc_value * const valueP) {
+
+    XMLRPC_ASSERT_ENV_OK(envP);
+    XMLRPC_ASSERT_VALUE_OK(structP);
+    XMLRPC_ASSERT_VALUE_OK(keyvalP);
+    XMLRPC_ASSERT_VALUE_OK(valueP);
+
+    if (structP->_type != XMLRPC_TYPE_STRUCT)
+        xmlrpc_env_set_fault(envP, XMLRPC_TYPE_ERROR,
+                             "Value is not a struct");
+    else if (keyvalP->_type != XMLRPC_TYPE_STRING)
+        xmlrpc_env_set_fault(envP, XMLRPC_TYPE_ERROR,
+                             "Key value is not a string");
+    else {
+        const char * const key =
+            XMLRPC_MEMBLOCK_CONTENTS(char, &keyvalP->_block);
+        size_t const keyLen =
+            XMLRPC_MEMBLOCK_SIZE(char, &keyvalP->_block) - 1;
+
+        bool found;
+        unsigned int index;
+
+        findMember(structP, key, keyLen, &found, &index);
+
+        if (found)
+            changeMemberValue(structP, index, valueP);
+        else
+            addNewMember(envP, structP, keyvalP, valueP);
+    }
 }
 
 
@@ -588,3 +617,31 @@ xmlrpc_struct_get_key_and_value(xmlrpc_env *    const envP,
         *valueP = NULL;
     }
 }
+
+
+
+/* Copyright (C) 2001 by First Peer, Inc. All rights reserved.
+**
+** Redistribution and use in source and binary forms, with or without
+** modification, are permitted provided that the following conditions
+** are met:
+** 1. Redistributions of source code must retain the above copyright
+**    notice, this list of conditions and the following disclaimer.
+** 2. Redistributions in binary form must reproduce the above copyright
+**    notice, this list of conditions and the following disclaimer in the
+**    documentation and/or other materials provided with the distribution.
+** 3. The name of the author may not be used to endorse or promote products
+**    derived from this software without specific prior written permission. 
+**  
+** THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
+** ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+** ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
+** FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+** DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+** OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+** HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+** LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+** OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+** SUCH DAMAGE. */
+
